IF NOT EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = N'TransformEntityDimProc')
	BEGIN
		EXECUTE ('CREATE PROCEDURE dbo.[TransformEntityDimProc] @WaterMark xml AS RETURN 1')
	END
GO

ALTER Procedure [dbo].[TransformEntityDimProc](@WaterMark xml)
as 
begin 
set nocount on 

      declare @WM datetime
            ,@MaxWaterMark datetime
            ,@BatchId int
            ,@Inserted int
            ,@err int
            ,@Updated int
     
      declare @nullguid uniqueidentifier
      set @nullguid = '00000000-0000-0000-0000-000000000000'
      
      select * into #tempTable from etl.ShredWaterMark(@WaterMark)

      select @WM=WaterMark, @BatchId=BatchId
      from #tempTable
      where WarehouseEntityName='MTV_System$Entity'
            and WaterMarkType='DateTime'

      SET  @MaxWaterMark= GETUTCDATE()
      EXEC @err = dbo.InferDatasourceDimProc
                @sourceTableName    = 'Inbound.BaseManagedEntity',
                @columnName         = 'DataSourceId',
                @filterColumnName   = 'DWTimeStamp',
                @minTimeStamp       = @WM,
                @maxTimeStamp       = @MaxWaterMark,
                @batchId            = @BatchId

    ;WITH TopRecordFromSource (
            BaseManagedEntityId,
            DisplayName,
            IsDeleted,
            SourceId,
            TimeAdded,
            LastModifiedBy,
            LastModified,
        RowNumber
    ) AS (
    SELECT
            BaseManagedEntityId,
            DisplayName,
            IsDeleted,
            source.DataSourceId as SourceId,
            TimeAdded,
            LastModifiedBy,
            LastModified,
            ROW_NUMBER() OVER (PARTITION BY source.DatasourceId, BaseManagedEntityId ORDER BY DWTimeStamp DESC, LastModified DESC) as RowNumber
    FROM    inbound.BaseManagedEntity source
    LEFT JOIN Staging.EntityTypeExceptionList exList ON
            source.DatasourceId = ISNULL(exList.DatasourceId, source.DatasourceId) -- if sourceid is null, then exclude from all sources
        and source.BaseManagedTypeId = exList.BaseManagedTypeId
    WHERE   source.DWTimestamp >= @WM
        AND source.DWTimestamp < @MaxWaterMark
        AND exList.BaseManagedTypeId IS NULL
    )
    SELECT  *
    INTO    #transformTemp1
    FROM    TopRecordFromSource source
    WHERE   source.RowNumber = 1

    CREATE CLUSTERED INDEX CI0_TxEntityDim ON #transformTemp1 (SourceId, BaseManagedEntityId)

      begin tran                
      Update dest
            set
                  DisplayName       = source.[DisplayName],
                  IsDeleted         = source.IsDeleted,
                  TimeAdded         = source.TimeAdded,
                  LastModifiedBy  = source.LastModifiedBy,
                  LastModified      = source.LastModified,
                  UpdatedBatchId  = @BatchId
      from #transformTemp1 source
      inner join dbo.EntityDim dest on(source.BaseManagedEntityId=dest.BaseManagedEntityId)
                        and (source.SourceId = dest.SourceId)

      select @Updated = @@RowCount, @err = @@error

      if(@err<>0)
      begin
            raiserror('Failed to update EntityDim Dimension', 16,1)
            return
      end

      insert into dbo.EntityDim(BaseManagedEntityId,DisplayName,IsDeleted,SourceId,TimeAdded,
                                                LastModifiedBy,LastModified,InsertedBatchId,UpdatedBatchId)
      select source.BaseManagedEntityId,
                  source.DisplayName,
            source.IsDeleted,
            source.SourceId,
            source.TimeAdded,
            source.LastModifiedBy,
            source.LastModified,
            @BatchId,
            0
      from #transformTemp1 source 
            left join dbo.EntityDim dest on (source.BaseManagedEntityId=dest.BaseManagedEntityId)
                                                      and (source.SourceId = dest.SourceId)
      where dest.BaseManagedEntityId is null and dest.SourceId is null

      select @Inserted=@@RowCount,@err=@@error

      if(@err<>0)
      begin
            raiserror('Failed to insert into EntityDim Dimension', 16,1)
            return
      end
      commit tran

      update #tempTable set WaterMark=convert(nvarchar(64),IsNull(@MaxWaterMark,WaterMark),109) where WarehouseEntityName='MTV_System$Entity' and WaterMarkType='DateTime'

      if(@err<>0)
      begin
            raiserror('Failed to update set watermark', 16,1)
            return
      end

      select @WaterMark = (select ModuleName as "@ModuleName",
            ProcessName as "@ProcessName",
            @BatchId as "@BatchId",
            (select distinct WarehouseEntityName as "@WarehouseEntityName",
                  WarehouseEntityTypeName as "@WarehouseEntityTypeName",
                  EntityGuid as "@EntityGuid",
                  case WarehouseEntityTypeName
                        when 'Inbound' then 'DateTime'
                        else 'BatchId' end as "@WaterMarkType",
                  case WarehouseEntityTypeName
                        when 'Inbound' then cast(WaterMark as nvarchar(64))
                        else cast(@BatchId as nvarchar(64)) end as "@WaterMark"
                  from #tempTable
                  for xml path('Entity'),type)
            from (select distinct ModuleName, ProcessName from #tempTable) a
            for xml path('Module'),type)

      select @WaterMark as WaterMark, @BatchId as BatchId, @Updated as UpdatedRowCount, @Inserted as InsertedRowCount

cleanup:
      if(OBJECT_ID('tempdb..#transformTemp1') is not null) drop table #transformTemp1

      if object_id('tempdb..#tempTable') is not null
      begin
            drop table #tempTable
      end

set nocount off
end

