if exists (select 'x' from sys.columns where object_id = OBJECT_ID('FIMApprovalResponseDim') and name = 'EntityDimKey' and is_nullable = 0)
begin
    exec('alter table dbo.FIMApprovalResponseDim alter column EntityDimKey int null');
end
go

if not exists (select 'x' from sys.procedures where name = 'TransformFIMApprovalResponseDimProc')
begin
    exec('create procedure dbo.TransformFIMApprovalResponseDimProc(@WaterMark XML) as begin return; end');
end
go


ALTER PROCEDURE [dbo].[TransformFIMApprovalResponseDimProc](@WaterMark XML)
AS BEGIN 
SET NOCOUNT ON 

/*
* NOTE:
* This is manually edited to fit the needs of FIM reporting.
* The auto-generated transform was manually edited to remove dependency on EntityDim table
* to support exclusion list functionality.
*/

    DECLARE @errorNumber INT, @errorSeverity INT, @errorState INT, @errorLine INT, @errorProcedure NVARCHAR(256), @errorMessage NVARCHAR(MAX), @task NVARCHAR(512)
    DECLARE @BatchId INT, @Updated INT = 0, @Inserted INT = 0, @retval INT = -1, @err INT = 0, @startTranCount INT = @@TRANCOUNT, @source1WM DATETIME, @source1MaxWM DATETIME

    BEGIN TRY
    SELECT @task = 'Shredding WaterMark'
    SELECT * INTO #tempTable FROM etl.ShredWaterMark(@WaterMark)

    SELECT @task = 'Reading WaterMark for @source1WM, @BatchId and @source1MaxWM'
    SELECT  @source1WM = WaterMark, @BatchId = BatchId, @source1MaxWM = MaxWaterMark 
    FROM    #tempTable
    WHERE   WarehouseEntityName='MTV_FIMDW$FIMApprovalResponse'
        AND WaterMarkType='DateTime'

    SET @source1MaxWM = GETUTCDATE()

    BEGIN TRANSACTION
    SELECT @task = 'Infer DatasourceDim'
    EXEC @err = dbo.InferDatasourceDimProc
        @sourceTableName    = 'inbound.MTV_FIMDW$FIMApprovalResponse',
        @columnName         = 'DataSourceId',
        @filterColumnName   = 'DWTimeStamp',
        @minTimeStamp       = @source1WM,
        @maxTimeStamp       = @source1MaxWM,
        @batchId            = @BatchId

    SELECT @task = 'Preparing #transformTemp1'
    ;WITH TopRecordFromSource (
        
  [FIMDW.FIMApprovalResponse!FIMObjectID],
        
        BaseManagedEntityId,
        DatasourceId,
        RowNumber
    ) AS (
    SELECT
        
    source.[FIMDW.FIMApprovalResponse!FIMObjectID],
        
        BaseManagedEntityId,
        DatasourceId,
        ROW_NUMBER() OVER (PARTITION BY DatasourceId,  BaseManagedEntityId ORDER BY LastModified DESC, DWTimestamp DESC) as RowNumber
    FROM    inbound.MTV_FIMDW$FIMApprovalResponse source
    WHERE   source.DWTimestamp >= @source1WM
        AND source.DWTimestamp < @source1MaxWM
        AND source.[FIMDW.FIMApprovalResponse] = 1
    )
    SELECT  *
    INTO    #transformTemp1
    FROM    TopRecordFromSource source
    WHERE   source.RowNumber = 1

    SELECT @task = 'Adding Clustered Index to #transformTemp1'
    DECLARE @temp UNIQUEIDENTIFIER = NEWID()
    DECLARE @t NVARCHAR(MAX) = 'CREATE CLUSTERED INDEX [CI1_' + CAST(@temp AS VARCHAR(64)) + '] ON #transformTemp1(DatasourceId, BaseManagedEntityId)'
    EXEC(@t)

    SELECT @task = 'Updating Dimension'
    UPDATE dest SET
        
    [FIMObjectID] = source.[FIMDW.FIMApprovalResponse!FIMObjectID],
        [SourceId] = source.[DatasourceId],
        [UpdatedBatchId] = @BatchId
    FROM #transformTemp1 source   
    INNER JOIN dbo.FIMApprovalResponseDimvw dest ON (source.BaseManagedEntityId = dest.BaseManagedEntityId AND source.DatasourceId = dest.SourceId)
    
    WHERE dest.BaseManagedEntityId is not null

    SELECT @Updated=@@RowCount,@err=@@error

    IF(@err<>0)
    BEGIN
        RAISERROR('Failed to update FIMApprovalResponseDim Dimension', 16,1)
        ROLLBACK TRAN
        RETURN
    END

    SELECT @task = 'Inserting into Dimension'
    INSERT INTO dbo.FIMApprovalResponseDimvw (
        [BaseManagedEntityId],
        [EntityDimKey],
    [FIMObjectID],
        [IsDeleted],
        [SourceId],
        [InsertedBatchId],
        [UpdatedBatchId]
    )
    SELECT DISTINCT
        source.BaseManagedEntityId,
        null as EntityDimKey,
        source.[FIMDW.FIMApprovalResponse!FIMObjectID],
        0 as [IsDeleted],
        source.[DatasourceId],
        @BatchId,
        0
    FROM #transformTemp1 source    
    LEFT JOIN dbo.FIMApprovalResponseDimvw dest ON (source.BaseManagedEntityId = dest.BaseManagedEntityId AND source.DatasourceId = dest.SourceId)
    
    WHERE dest.BaseManagedEntityId is null

    select @Inserted=@@RowCount,@err=@@error

    IF(@err<>0)
    BEGIN
        RAISERROR('Failed to insert into FIMApprovalResponseDim Dimension', 16,1)
        ROLLBACK TRAN
        RETURN
    END
 
    SELECT @task = 'Update source to move early arriving records to next batch'
    UPDATE source SET
        DWTimestamp = @source1MaxWM,
        RejectedCount = ISNULL(source.RejectedCount, 0) + 1,
        BatchId = @BatchId
    FROM inbound.MTV_FIMDW$FIMApprovalResponse source
    LEFT JOIN dbo.FIMApprovalResponseDimvw baseEntityDim ON (source.BaseManagedEntityId = baseEntityDim.BaseManagedEntityId AND source.DatasourceId = baseEntityDim.SourceId)
    WHERE source.DWTimestamp >= @source1WM AND source.DWTimestamp < @source1MaxWM
    AND baseEntityDim.BaseManagedEntityId is null


    IF(@err <> 0)
    BEGIN
        RAISERROR('Failed to update inbound table for Dimension', 16,1)      
        ROLLBACK TRAN  
        RETURN
    END

    COMMIT TRANSACTION

    SELECT @task = 'Update #tempTable for WaterMark'
    UPDATE #tempTable SET WaterMark = CONVERT(nvarchar(64), ISNULL(@source1MaxWM,WaterMark),109) 
    WHERE WarehouseEntityName='MTV_FIMDW$FIMApprovalResponse' AND WaterMarkType='DateTime'

    IF(@err <> 0)
    BEGIN
        RAISERROR('Failed to update set watermark', 16,1)        
        RETURN
    END

    SELECT @task = 'Building output Watermark XML'
    SELECT @WaterMark =
        (SELECT ModuleName AS "@ModuleName", ProcessName AS "@ProcessName", @BatchId AS "@BatchId",
        (SELECT DISTINCT WarehouseEntityName AS "@WarehouseEntityName", WarehouseEntityTypeName AS "@WarehouseEntityTypeName", EntityGuid AS "@EntityGuid",
            CASE WarehouseEntityTypeName WHEN 'Inbound' THEN 'DateTime' WHEN 'Enumeration' THEN 'DateTime' ELSE 'BatchId' END AS "@WaterMarkType",
            CASE WarehouseEntityTypeName WHEN 'Inbound' THEN CONVERT(nvarchar(64), WaterMark, 109) WHEN 'Enumeration' THEN CONVERT(nvarchar(64), WaterMark, 109) ELSE CAST(@BatchId AS nvarchar(64)) END AS "@WaterMark"
            FROM #tempTable
            FOR xml path('Entity'),type)
        FROM (SELECT DISTINCT ModuleName, ProcessName FROM #tempTable) a
        FOR xml path('Module'),type)

    SELECT @WaterMark AS WaterMark, @BatchId AS BatchId, @Updated AS UpdatedRowCount, @Inserted AS InsertedRowCount

    SELECT @task = 'Cleaning up by deleting temp tables no longer needed'
    IF OBJECT_ID('tempdb..#tempTable') IS NOT NULL DROP TABLE #tempTable
    IF OBJECT_ID('tempdb..#transformTemp1') IS NOT NULL DROP TABLE #transformTemp1
    END TRY
    BEGIN CATCH
    DECLARE @errorFmt       VARCHAR(256)

    SELECT  @errorFmt = N'ErrorNumber="%d" Message="%s" Severity="%d" State="%d" ProcedureName="%s" LineNumber="%d" Task="%s"',
            @errorNumber = ERROR_NUMBER(), @errorMessage = ERROR_MESSAGE(), @errorSeverity = ERROR_SEVERITY(), @errorState = ERROR_STATE(), @errorLine = ERROR_LINE(), @errorProcedure = ERROR_PROCEDURE() 

    IF(@@TRANCOUNT > @startTranCount) ROLLBACK TRANSACTION
    
    IF OBJECT_ID('tempdb..#tempTable') IS NOT NULL DROP TABLE #tempTable
    IF OBJECT_ID('tempdb..#transformTemp1') IS NOT NULL DROP TABLE #transformTemp1

    RAISERROR (@errorFmt, 18, @errorState, @errorNumber, @errorMessage, @errorSeverity, @errorState, @errorProcedure, @errorLine, @task)
    RETURN -1
    END CATCH

SET NOCOUNT OFF
END
GO


