if object_id ( 'etl.GetMetadata', 'p' ) is not null 
    drop procedure etl.GetMetadata
go
create procedure etl.GetMetadata							   
/*
	Gets the list of all modules, relational entities, jobs
	Usage: etl.GetMetadata 
*/
as
begin
set nocount on 

	select distinct SourceId, SourceName, SourceGuid, SourceTypeName
	from etl.Source S join etl.SourceType st on (S.SourceTypeId = st.SourceTypeId)
		
	select distinct E.WarehouseEntityName,ET.WarehouseEntityTypeName, E.EntityGuid, E.SourceId	
	from etl.WarehouseEntity E 
					inner join etl.WarehouseEntityType ET on (E.WarehouseEntityTypeId = ET.WarehouseEntityTypeId)					

	/*Get all the modules for a given job*/			
	select distinct M.ModuleName, M.ModuleGuid, P.ProcessName, M.Description, MT.ModuleTypeName, J.JobName, E.WarehouseEntityName
		  , ET.WarehouseEntityTypeName,E.EntityGuid, cast(etl.GetDependencies(M.ModuleId, null) as nvarchar(max)) as Dependencies
	from etl.WarehouseModule M 
				join etl.Job J on (M.JobId = J.JobId)
				join etl.Process P on (M.ProcessId = P.ProcessId)	
				join etl.WarehouseModuleType MT on (M.ModuleTypeId = MT.ModuleTypeId)
				join etl.WarehouseEntity E on (M.TargetWarehouseEntityId = E.WarehouseEntityId)
				join etl.WarehouseEntityType ET on (E.WarehouseEntityTypeId = ET.WarehouseEntityTypeId)
				join etl.WarehouseModuleDependency MD on (M.ModuleId = MD.ModuleId)

    EXEC ETL.GetConfiguration
	return 0

set nocount on 
end

go

-- Adding this procedure to support FIM scenarios around grooming SM instances based on DW watermarks
if not exists(select 'x' from sys.procedures where name = 'GetDWSourceInfo')
begin
    exec('create procedure etl.GetDWSourceInfo(@sourceId int = null, @sourceGuid uniqueidentifier = null, @sourceName nvarchar(512) = null, @sourceTypeId int = null) as select 0 as SourceId where 1=2');
end
go

/*
exec etl.GetDWSourceInfo
    --@sourceName = 'SCDW',
    --@sourceTypeId = 2,
    @sourceGuid = 'CD54EB80-9D73-7390-D018-DF1A7325FEEE'
*/
alter procedure etl.GetDWSourceInfo(@sourceId int = null, @sourceGuid uniqueidentifier = null, @sourceName nvarchar(512) = null, @sourceTypeId int = null)
as begin
    select SourceId, SourceGuid, SourceName, SourceTypeId
    from etl.Source as src
    where SourceId = ISNULL(@sourceId, src.SourceId)
        and SourceGuid = ISNULL(@sourceGuid, src.SourceGuid)
        and SourceName = ISNULL(@sourceName, src.SourceName)
        and SourceTypeId = ISNULL(@sourceTypeId, src.SourceTypeId)
end
go


