---------------------------------------------------------------------
-- This adds a staging step to all datasources.
-- The step should be added when the System.Library MP is processed.
---------------------------------------------------------------------
if (select COUNT(*) from Staging.RawTableExtract where RawSourceTableName = N'EnumType' and RawTargetTableName = N'EnumType') = 0
begin
  insert Staging.RawTableExtract (
   RawSourceSchemaName
  ,RawSourceTableName
  ,RawTargetSchemaName
  ,RawTargetTableName
  ,MPName
  ,DatasourceId
  ,LastModified)
  values (
   'dbo'
  ,'EnumType'
  ,'inbound'
  ,'EnumType'
  ,'System.Library'
  ,null
  ,GETUTCDATE())
  
end
go

if (select COUNT(*) from Staging.RawTableExtract where RawSourceTableName = N'ManagedType' and RawTargetTableName = N'ManagedType') = 0
begin
  insert Staging.RawTableExtract (
   RawSourceSchemaName
  ,RawSourceTableName
  ,RawTargetSchemaName
  ,RawTargetTableName
  ,MPName
  ,DatasourceId
  ,LastModified)
  values (
   'dbo'
  ,'ManagedType'
  ,'inbound'
  ,'ManagedType'
  ,'System.Library'
  ,null
  ,GETUTCDATE())
  
end
go

if (select COUNT(*) from Staging.RawTableExtract where RawSourceTableName = N'RelationshipType' and RawTargetTableName = N'RelationshipType') = 0
begin
  insert Staging.RawTableExtract (
   RawSourceSchemaName
  ,RawSourceTableName
  ,RawTargetSchemaName
  ,RawTargetTableName
  ,MPName
  ,DatasourceId
  ,LastModified)
  values (
   'dbo'
  ,'RelationshipType'
  ,'inbound'
  ,'RelationshipType'
  ,'System.Library'
  ,null
  ,GETUTCDATE())
  
end
go

if (select COUNT(*) from Staging.RawTableExtract where RawSourceTableName = N'BaseManagedEntity' and RawTargetTableName = N'BaseManagedEntity') = 0
begin
  insert Staging.RawTableExtract (
   RawSourceSchemaName
  ,RawSourceTableName
  ,RawTargetSchemaName
  ,RawTargetTableName
  ,MPName
  ,DatasourceId
  ,LastModified)
  values (
   'dbo'
  ,'BaseManagedEntityExtractView'
  ,'inbound'
  ,'BaseManagedEntity'
  ,'System.Library'
  ,null
  ,GETUTCDATE())
  
end
go

if (select COUNT(*) from Staging.RawTableExtract where RawSourceTableName = N'TypedManagedEntity' and RawTargetTableName = N'TypedManagedEntity') = 0
begin
  insert Staging.RawTableExtract (
   RawSourceSchemaName
  ,RawSourceTableName
  ,RawTargetSchemaName
  ,RawTargetTableName
  ,MPName
  ,DatasourceId
  ,LastModified)
  values (
   'dbo'
  ,'TypedManagedEntity'
  ,'inbound'
  ,'TypedManagedEntity'
  ,'System.Library'
  ,null
  ,GETUTCDATE())
  
end
go

-- If they are upgrading, change the Source Name for Relationship
-- so that Extract will use a new view that contains SourceTypeId and TargetTypeId
-- If the row is not there to be updated, the subsequent If statement will insert it

UPDATE Staging.RawTableExtract SET RawSourceTableName = 'BaseManagedEntityExtractView' WHERE RawTargetTableName = 'BaseManagedEntity'
UPDATE Staging.RawTableExtract SET RawSourceTableName = 'RelationshipExtractView' WHERE RawTargetTableName = 'Relationship'
GO

if (select COUNT(*) from Staging.RawTableExtract where RawSourceTableName = N'RelationshipExtractView' and RawTargetTableName = N'Relationship') = 0
begin
  insert Staging.RawTableExtract (
   RawSourceSchemaName
  ,RawSourceTableName
  ,RawTargetSchemaName
  ,RawTargetTableName
  ,MPName
  ,DatasourceId
  ,LastModified)
  values (
   'dbo'
  ,'RelationshipExtractView'
  ,'inbound'
  ,'Relationship'
  ,'System.Library'
  ,null
  ,GETUTCDATE())
  
end
go

if (select COUNT(*) from Staging.RawTableExtract where RawSourceTableName = N'DisplayStringView' and RawTargetTableName = N'DisplayString') = 0
begin
  insert Staging.RawTableExtract (
   RawSourceSchemaName
  ,RawSourceTableName
  ,RawTargetSchemaName
  ,RawTargetTableName
  ,MPName
  ,DatasourceId
  ,LastModified)
  values (
   'dbo'
  ,'DisplayStringView'
  ,'inbound'
  ,'DisplayString'
  ,'System.Library'
  ,null
  ,GETUTCDATE())
end
go

if not exists(select 'x' from Staging.EntityTypeExceptionList where TypeName = 'System.Compliance.ManagedEntityResult')
begin
    insert into Staging.EntityTypeExceptionList (DatasourceId, BaseManagedTypeId, TypeName)
    values(null, '4C956017-0387-A473-0C87-1C8EEC2D4A7D', 'System.Compliance.ManagedEntityResult'),
          (null, 'E84A61C6-09B3-8CC7-E760-41E0B0BC9862', 'System.Compliance.ManagedEntityResult')
end
go

if not exists(select 'x' from Staging.EntityTypeExceptionList where TypeName = 'FIMDW.Approval')
begin
    insert into Staging.EntityTypeExceptionList (DatasourceId, BaseManagedTypeId, TypeName)
    values(null, '53A21913-7848-8271-AC40-C0B38EFE8EED', 'FIMDW.FIMApproval')

end
go

if not exists(select 'x' from Staging.EntityTypeExceptionList where TypeName = 'FIMDW.FIMApprovalResponse')
begin
    insert into Staging.EntityTypeExceptionList (DatasourceId, BaseManagedTypeId, TypeName)
    values(null, '9EE25506-4A33-FCB7-DFBE-DF72965B16BC', 'FIMDW.FIMApprovalResponse')

end
go

if not exists(select 'x' from Staging.EntityTypeExceptionList where TypeName = 'FIMDW.FIMRequest')
begin
    insert into Staging.EntityTypeExceptionList (DatasourceId, BaseManagedTypeId, TypeName)
    values(null, 'EF7358DB-3D36-D0CE-A16C-6E1B7B5C6926', 'FIMDW.FIMRequest')

end
go

if not exists(select 'x' from Staging.EntityTypeExceptionList where TypeName = 'FIMDW.FIMRequestTargetDetail')
begin
    insert into Staging.EntityTypeExceptionList (DatasourceId, BaseManagedTypeId, TypeName)
    values(null, '9B235429-B989-97F0-6F6B-A573AEBB76AC', 'FIMDW.FIMRequestTargetDetail')

end
go


-- create synonyms for staging/inbound objects in the repository
EXEC Staging.p_MakeSynonymInDW N'EntityTypeExceptionList', N'Staging'





