IF OBJECT_ID(N'Staging.p_MakeSynonymInDW') IS NOT NULL 
  DROP PROCEDURE Staging.p_MakeSynonymInDW
go

CREATE PROCEDURE Staging.p_MakeSynonymInDW
(
  @ObjectName	sysname,
  @SchemaName sysname = N'inbound'
)
AS
BEGIN 
SET NOCOUNT ON

-- We need to look at the Resource Store View to get the name of the Repository DB
-- This view will always be in the Configuration DB on the DW server.  If we don't find it, 
-- we assume this script is still running in the Repository DB instead of Config DB, and there is
-- no need to create the synonym until Setup is changed.
 
IF NOT EXISTS (SELECT 1 FROM INFORMATION_SCHEMA.VIEWS WHERE TABLE_NAME = N'MTV_Microsoft$SystemCenter$ResourceAccessLayer$SqlResourceStore')
	RETURN 0

DECLARE @DW_DB sysname
DECLARE @Config_DB sysname
DECLARE @Command nvarchar(max)


SET @Config_DB = DB_NAME()

SELECT @DW_DB = DataService_98B2DDF9_D9FD_9297_85D3_FCF36F1D016B
	FROM dbo.MTV_Microsoft$SystemCenter$ResourceAccessLayer$SqlResourceStore
	WHERE Name_188E1D94_94DB_6A6B_3897_D7C2F6DFDBB6 = 'Ral.SqlResourceStore.Repository'

SET @Command = N'EXEC [' + @DW_DB + N'].dbo.p_MakeSynonym ''[' + @Config_DB + N']'', ''' + @SchemaName + N''', ''' + @ObjectName + N''''
EXEC (@Command)

END
GO

IF OBJECT_ID(N'Staging.p_MakeViewForAbstractType') IS NOT NULL 
  DROP PROCEDURE Staging.p_MakeViewForAbstractType
go

CREATE PROCEDURE Staging.p_MakeViewForAbstractType
(
  @ManagedTypeId uniqueidentifier
)
AS
BEGIN 
SET NOCOUNT ON
DECLARE @ViewName sysname
DECLARE @ClassName sysname
DECLARE @Command nvarchar(max)
DECLARE @ColumnList nvarchar(max)
DECLARE @FoundDerivedTypeWithViewOrTable bit
DECLARE @IsAbstract bit
DECLARE @BaseManagedType uniqueidentifier
DECLARE @FirstTime bit
DECLARE @DerivedTypeId uniqueidentifier
DECLARE @ManagedTypeIdString nvarchar(40)

DECLARE @TablesUsed TABLE (TableName nvarchar(255))

SET @ManagedTypeIdString = CONVERT(nvarchar(40), @ManagedTypeId)

SELECT  @ClassName = TypeName, @IsAbstract = IsAbstract, @BaseManagedType = BaseManagedTypeId
	FROM Staging.ManagedType 
	WHERE ManagedTypeId = @ManagedTypeId

IF @IsAbstract = 0
    RETURN 0

SET @FoundDerivedTypeWithViewOrTable = 0
SET @ViewName = Staging.fn_DeterministicObjectName(N'MTV_', @ClassName)

-- Build Column List using fn_TypeAndAbstractParents and ManagedTypeProperty table
SET @ColumnList = N''
-- Get the generic properties that are in all the staging tables
SELECT @ColumnList = @ColumnList + N', [' + TargetColumn + N']' FROM Staging.fn_GetStagingTableColumns ( '00000000-0000-0000-0000-000000000000')  ORDER BY ManagedTypePropertyId
-- Now get the type specific properties
SELECT @ColumnList = @ColumnList + N', [' + TypeName + N'!' + ManagedTypePropertyName + N']'  from Staging.ManagedTypeProperty MTP 
	JOIN Staging.ManagedType MT ON MT.ManagedTypeId = MTP.ManagedTypeId where  ColumnName <> N'DisplayName' AND MTP.ManagedTypeId IN 
	(SELECT ManagedTypeId FROM Staging.fn_TypeAndAbstractParents(@ManagedTypeId))
SET @ColumnList = SUBSTRING(@ColumnList, 2, LEN(@ColumnList))

-- Build Root of Command using CREATE VIEW or ALTER VIEW depending on whether View is in
-- INFORMATION_SCHEMA.VIEWS already.
IF EXISTS (SELECT 1 FROM INFORMATION_SCHEMA.VIEWS WHERE TABLE_NAME = @ViewName)
	BEGIN
	SET @FirstTime = 0
	SET @Command = N'ALTER VIEW inbound.' + @ViewName + N' '
	END
ELSE
	BEGIN
	SET @FirstTime = 1
	SET @Command = N'CREATE VIEW inbound.' + @ViewName + N' '
	END

-- Iterate over the concrete types which inherit from this abstract type
DECLARE @DerivedTypeViewName sysname
DECLARE @DerivedTypeName sysname
DECLARE DerivedTypeCursor CURSOR LOCAL FORWARD_ONLY READ_ONLY FOR 
	SELECT DISTINCT MT.ManagedTypeId, MT.ManagedTypeViewName, MT.TypeName 
	FROM Staging.fn_DerivedManagedTypes(@ManagedTypeId) FN JOIN Staging.ManagedType MT
		ON FN.DerivedManagedTypeId = MT.ManagedTypeId
		WHERE FN.IsAbstract = 0
OPEN DerivedTypeCursor
FETCH NEXT FROM DerivedTypeCursor INTO @DerivedTypeId, @DerivedTypeViewName, @DerivedTypeName
WHILE @@FETCH_STATUS = 0
	BEGIN
		-- For Abstract Types, the View Name in Staging.ManagedType table is NULL, but we know what it should be
		IF @DerivedTypeViewName IS NULL
		    SET @DerivedTypeViewName = Staging.fn_DeterministicObjectName(N'MTV_', @DerivedTypeName)

		IF EXISTS (SELECT 1 FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = @DerivedTypeViewName) AND 
			NOT EXISTS (SELECT 1 FROM @TablesUsed WHERE TableName = @DerivedTypeViewName)
			BEGIN
			IF @FoundDerivedTypeWithViewOrTable = 1
			   SET @Command = @Command + N' UNION ALL 
	SELECT ' + @ColumnList + N' 
		FROM inbound.' + @DerivedTypeViewName
			ELSE
			   SET @Command = @Command + N' AS 
	SELECT ' + @ColumnList + N' 
		FROM inbound.' + @DerivedTypeViewName
			SET @FoundDerivedTypeWithViewOrTable = 1
			INSERT INTO @TablesUsed (TableName) VALUES (@DerivedTypeViewName)
			END
	
	FETCH NEXT FROM DerivedTypeCursor INTO @DerivedTypeId, @DerivedTypeViewName, @DerivedTypeName
	END
CLOSE DerivedTypeCursor
DEALLOCATE DerivedTypeCursor

-- Make View include all singleton instances derived from this type in unsealed MP's.
-- If there are derived concrete types, then that is basically the entire view.  The important things
-- to get from BME are the DisplayName for the singleton instance and any metadata columns that
-- might be used in automatically generated transforms.
IF @FoundDerivedTypeWithViewOrTable = 0
	BEGIN
	-- Make a dummy view that has columns of correct data type for the abstract type
	-- but that will return no rows.  If the Transform is called before a DerivedType is Added
	-- it should just do nothing and return.
	SET @Command = @Command + N' AS 
	SELECT '
	END
	ELSE
		SET @Command = @Command + N' UNION ALL  
	SELECT '
	
-- First add constants and column names for the generic properties that are in all staging tables
SELECT @Command = @Command + ColumnName  + N' AS [' + TargetColumn + N'] ,'
	FROM Staging.fn_GetStagingTableColumns ( '00000000-0000-0000-0000-000000000000') ORDER BY ManagedTypePropertyId
-- Now add constants and column names for the type specific properties
SELECT @Command = @Command + [Staging].[fn_MapDatatypeToConstant](PropertyType) + 
	N' AS [' + TypeName + N'!' + ManagedTypePropertyName + N'] ,'  from Staging.ManagedTypeProperty MTP 
	JOIN Staging.ManagedType MT ON MT.ManagedTypeId = MTP.ManagedTypeId where  ColumnName <> N'DisplayName' AND MTP.ManagedTypeId IN 
	(SELECT ManagedTypeId FROM Staging.fn_TypeAndAbstractParents(@ManagedTypeId)) ORDER BY ManagedTypePropertyId
-- Now trim off the final comma and add a where clause for inbound BME table.  Restrict to UNSEALED singletons, since the singletons in
-- sealed MP's would already be picked up by the initial loop over derived classes.
SET @Command = SUBSTRING(@Command, 1, len(@Command) - 1) + N' 
  FROM inbound.BaseManagedEntity WHERE 
 BaseManagedTypeId IN (SELECT ManagedTypeId FROM inbound.ManagedType WHERE IsSealed = 0 AND IsSingleton = 1 AND BaseManagedTypeId in ( 
 SELECT DISTINCT MT.ManagedTypeId FROM Staging.fn_DerivedManagedTypes(''' + @ManagedTypeIdString + ''') FN 
 JOIN Staging.ManagedType MT  ON FN.DerivedManagedTypeId = MT.ManagedTypeId  WHERE FN.IsAbstract = 1 ))'

-- EXEC command to create the view if we found a DerivedType with a View or Table
EXEC (@Command)

-- If we are creating the view in the DWCMDB then add a synonym in the SMDW database
EXEC Staging.p_MakeSynonymInDW @ViewName, N'inbound'

-- At some point we have to add synonyms for DisplayString and EnumType tables; do it when we are processing System.Entity for the first time
-- Taking out the @FirstTime check coz we added new synonyms in SP1 that wouldn't get executed. Besides these scripts are reentrant and handle cases where synonyms already exists
-- Which is the case for the rest of the ones. 
IF @ClassName = N'System.Entity'
	BEGIN
	EXEC Staging.p_MakeSynonymInDW N'EnumType', N'inbound'
	EXEC Staging.p_MakeSynonymInDW N'DisplayString', N'inbound'
	EXEC Staging.p_MakeSynonymInDW N'BaseManagedEntity', N'inbound'
	EXEC Staging.p_MakeSynonymInDW N'TypedManagedEntity', N'inbound'
	EXEC Staging.p_MakeSynonymInDW N'Relationship', N'inbound'
	EXEC Staging.p_MakeSynonymInDW N'ManagedType', N'inbound'
	EXEC Staging.p_MakeSynonymInDW N'RelationshipType', N'inbound'
	EXEC Staging.p_MakeSynonymInDW N'ManagementPack', N'inbound'
	EXEC Staging.p_MakeSynonymInDW N'fn_FirstLevelConcreteDerivedManagedTypes', N'Staging' 
	EXEC Staging.p_MakeSynonymInDW N'fn_DerivedManagedTypes', N'Staging'
	EXEC Staging.p_MakeSynonymInDW N'ManagedType', N'Staging'
	EXEC Staging.p_MakeSynonymInDW N'EntityTypeExceptionList', N'Staging'
	END

-- If we are based on another Abstract Type (true for everything except System.Entity) then
--  create or alter the view for our Base Type

IF @BaseManagedType IS NOT NULL
	EXEC Staging.p_MakeViewForAbstractType @BaseManagedType

RETURN 0

END

GO

IF OBJECT_ID(N'Staging.p_MakeViewForAbstractRelationshipType') IS NOT NULL 
  DROP PROCEDURE Staging.p_MakeViewForAbstractRelationshipType
go

CREATE PROCEDURE Staging.p_MakeViewForAbstractRelationshipType
(
  @RelationshipTypeId uniqueidentifier
)
AS
BEGIN 
SET NOCOUNT ON
DECLARE @Command nvarchar(max)
DECLARE @RelationshipTypeString nvarchar(40)
DECLARE @ViewName nvarchar(128)
DECLARE @RelationshipTypeName nvarchar(255)

SET @RelationshipTypeString = CONVERT(nvarchar(36), @RelationshipTypeId)
SELECT @RelationshipTypeName = RelationshipTypeName FROM Staging.RelationshipType
 WHERE RelationshipTypeId = @RelationshipTypeId
 
SET @ViewName  = Staging.fn_DeterministicObjectName(N'MTV_', @RelationshipTypeName)

-- Build Root of Command using CREATE VIEW or ALTER VIEW depending on whether View is in
-- INFORMATION_SCHEMA.VIEWS already.
IF EXISTS (SELECT 1 FROM INFORMATION_SCHEMA.VIEWS WHERE TABLE_NAME = @ViewName)
 BEGIN
 SET @Command = N'DROP VIEW inbound.' + @ViewName + N' '
 EXEC (@Command)
 END

SET @Command = N'CREATE VIEW inbound.' + @ViewName + N' AS
  select * from inbound.Relationship where RelationshipTypeId in
  (Select Distinct RelationshipTypeId from ' + N' [Staging].[fn_GetConcreteRelationshipTypeId] (''' + @RelationshipTypeString + '''))'
 
EXEC (@Command)

UPDATE Staging.RelationshipType SET RelationshipTypeViewName = @ViewName 
 WHERE RelationshipTypeId = @RelationshipTypeId
  
-- If we are creating the view in the DWCMDB then add a synonym in the SMDW database
EXEC Staging.p_MakeSynonymInDW @ViewName, N'inbound'

 
RETURN 0

END

GO

if object_id(N'Staging.AssociateMP') is not null
  drop procedure Staging.AssociateMP
go

create procedure Staging.AssociateMP
 @DatasourceId              uniqueidentifier
,@ManagementPackId          uniqueidentifier
,@Disassociate              bit                 = 0
as
begin
set nocount on
declare @error   int

if not exists (select * from Staging.ManagementPack where ManagementPackId = @ManagementPackId)
begin
  set @error = 778000002
  goto error
end

begin tran
if not exists (select * from Staging.Datasource where DatasourceId = @DatasourceId)
begin
  insert Staging.Datasource (DatasourceId, TimeAdded, LastModified)
  select @DatasourceId, getutcdate(), getutcdate()

  set @error = @@error
  if @error != 0 goto error
end

if not exists (select * from Staging.ExtractionMP where ManagementPackId = @ManagementPackId and DatasourceId = @DatasourceId)
begin
  insert Staging.ExtractionMP (ManagementPackId, IsDeleted, DatasourceId, TimeAdded, LastModified)
  select @ManagementPackId, @Disassociate, @DatasourceId, getutcdate(), getutcdate()

  set @error = @@error
  if @error != 0 goto error
end
else
begin
  update a set
   IsDeleted = @Disassociate
  ,LastModified = getutcdate()
  from Staging.ExtractionMP a
  where ManagementPackId = @ManagementPackId 
  and DatasourceId = @DatasourceId

  set @error = @@error
  if @error != 0 goto error
end

delete a
from Staging.ExtractionProperty a
join Staging.ExtractionType b on a.TypeId=b.TypeId
  and a.DatasourceId = b.DatasourceId
where b.ManagementPackId = @ManagementPackId
and   b.DatasourceId = @DatasourceId

set @error = @@error
if @error != 0 goto error

update a set
 a.IsDeleted = 1
from  Staging.ExtractionType a
where ManagementPackId = @ManagementPackId and DatasourceId = @DatasourceId

set @error = @@error
if @error != 0 goto error

if @Disassociate = 0
begin  
  update b set
   b.IsDeleted = 0
  from   Staging.ManagedType a
  join Staging.ExtractionType b on a.ManagedTypeId=b.TypeId
    and a.ManagementPackId=b.ManagementPackId
    and b.DatasourceId=@DatasourceId
  where  a.ManagementPackId = @ManagementPackId

  set @error = @@error
  if @error != 0 goto error

  insert Staging.ExtractionType (TypeId, 
                                 TypeName, 
                                 SourceViewName, 
                                 IsManagedType, 
                                 IsDeleted, 
                                 IsActive,
                                 ManagementPackId, 
                                 DatasourceId, 
                                 TimeAdded, 
                                 LastModified)
  select a.ManagedTypeId,  
         a.TypeName, 
         a.ManagedTypeViewName, 
         1, 
         0, 
         1,
         a.ManagementPackId, 
         @DatasourceId, 
         getutcdate(),
         getutcdate()
  from   Staging.ManagedType a
  left join Staging.ExtractionType b on a.ManagedTypeId=b.TypeId
    and a.ManagementPackId=b.ManagementPackId
    and b.DatasourceId=@DatasourceId
  where  a.ManagementPackId = @ManagementPackId
  and    b.TypeId is null

  set @error = @@error
  if @error != 0 goto error

  update b set
   b.IsDeleted = 0
  from Staging.ManagedTypeProperty a
  join Staging.ExtractionProperty b on a.ManagedTypePropertyId=b.PropertyId
    and a.ManagedTypeId=b.TypeId
    and b.DatasourceId=@DatasourceId
  where a.ManagementPackId = @ManagementPackId

  set @error = @@error
  if @error != 0 goto error

  insert Staging.ExtractionProperty (PropertyId,
                                     PropertyName,
                                     ColumnName,
                                     IsDeleted,
                                     IsActive,
                                     TypeId,
                                     DatasourceId,
                                     TimeAdded,
                                     LastModified)
  select a.ManagedTypePropertyId,
         a.ManagedTypePropertyName,
         a.ColumnName,
         0,
         1,
         a.ManagedTypeId,
         @DatasourceId,
         getutcdate(),
         getutcdate()
  from Staging.ManagedTypeProperty a
  left join Staging.ExtractionProperty b on a.ManagedTypePropertyId=b.PropertyId
    and a.ManagedTypeId=b.TypeId
    and b.DatasourceId=@DatasourceId
  where a.ManagementPackId = @ManagementPackId
  and   b.PropertyId is null

  set @error = @@error
  if @error != 0 goto error

  update b set
   b.IsDeleted = 0
  from   Staging.RelationshipType a
  join Staging.ExtractionType b on a.RelationshipTypeId=b.TypeId
    and a.ManagementPackId=b.ManagementPackId
    and b.DatasourceId=@DatasourceId
  where  a.ManagementPackId = @ManagementPackId

  set @error = @@error
  if @error != 0 goto error

  insert Staging.ExtractionType (TypeId, 
                                 TypeName, 
                                 SourceViewName, 
                                 IsManagedType, 
                                 IsDeleted, 
                                 IsActive,
                                 ManagementPackId, 
                                 DatasourceId, 
                                 TimeAdded, 
                                 LastModified)
  select a.RelationshipTypeId, 
         a.RelationshipTypeName, 
         a.RelationshipTypeViewName, 
         0, 
         0, 
         1,
         a.ManagementPackId, 
         @DatasourceId, 
         getutcdate(),
         getutcdate()
  from   Staging.RelationshipType a
  left join Staging.ExtractionType b on a.RelationshipTypeId=b.TypeId
    and a.ManagementPackId=b.ManagementPackId
    and b.DatasourceId=@DatasourceId
  where  a.ManagementPackId = @ManagementPackId
  and    b.TypeId is null

  set @error = @@error
  if @error != 0 goto error

  update b set
   b.IsDeleted = 0
  from Staging.RelationshipTypeProperty a
  join Staging.ExtractionProperty b on a.RelationshipPropertyId=b.PropertyId
    and a.RelationshipTypeId=b.TypeId
    and b.DatasourceId=@DatasourceId
  where a.ManagementPackId = @ManagementPackId

  set @error = @@error
  if @error != 0 goto error

  insert Staging.ExtractionProperty (PropertyId,
                                     PropertyName,
                                     ColumnName,
                                     IsDeleted,
                                     IsActive,
                                     TypeId,
                                     DatasourceId,
                                     TimeAdded,
                                     LastModified)
  select a.RelationshipPropertyId,
         a.RelationshipPropertyName,
         a.ColumnName,
         0,
         1,
         a.RelationshipTypeId,
         @DatasourceId,
         getutcdate(),
         getutcdate()
  from Staging.RelationshipTypeProperty a
  left join Staging.ExtractionProperty b on a.RelationshipPropertyId=b.PropertyId
    and a.RelationshipTypeId=b.TypeId
    and b.DatasourceId=@DatasourceId
  where a.ManagementPackId = @ManagementPackId
  and   b.PropertyId is null

  set @error = @@error
  if @error != 0 goto error
end

commit tran

error:
  if @error > 0 
  begin
    if XACT_STATE() != 0 rollback tran
    raiserror(@error, 16, 1)
  end

  return
end
go

if object_id(N'Staging.AlterStagingTableColumnsForRelationshipType') is not null
  drop procedure Staging.AlterStagingTableColumnsForRelationshipType
go

create procedure Staging.AlterStagingTableColumnsForRelationshipType
  @RelationshipTypeId     uniqueidentifier
as
begin
set nocount on

-- Name of the Source Table
declare @table nvarchar(128)
declare @column nvarchar(128)
declare @length int
declare @casesensitive bit
declare @command nvarchar(max)
select @table = RelationshipTypeViewName from Staging.RelationshipType where RelationshipTypeId = @RelationshipTypeId

declare @columns table (RelationshipTypePropertyId   uniqueidentifier, 
                        RelationshipTypeId           uniqueidentifier,
                        SourceTableName         nvarchar(128),
                        ColumnName              nvarchar(128), 
                        TargetColumn            nvarchar(128),
                        PropertyType            int, 
                        IsCaseSensitive         bit, 
                        Required                bit, 
                        PropertyLength          int, 
                        PropertyScale           int, 
                        DefaultValue            nvarchar(2000), 
                        MaxValue                int, 
                        MinValue                int)
insert @columns select * FROM Staging.fn_GetStagingTableColumns(@RelationshipTypeId)

-- Set up Cursor over columns that should be altered
--  e.g.  strings that have length increased, and non-string datatypes
--  that are changing to string.
DECLARE C2 CURSOR LOCAL FORWARD_ONLY READ_ONLY FOR
SELECT N'[' + cols.TargetColumn + N']', cols.PropertyLength, cols.IsCaseSensitive
    FROM @columns cols
     	      JOIN [INFORMATION_SCHEMA].[COLUMNS] INFO ON (INFO.COLUMN_NAME = cols.TargetColumn AND  TABLE_NAME = @table and TABLE_SCHEMA = N'inbound')
   WHERE   cols.PropertyType = 3 AND ((cols.PropertyLength > INFO.CHARACTER_MAXIMUM_LENGTH AND INFO.CHARACTER_MAXIMUM_LENGTH > 0) OR INFO.CHARACTER_MAXIMUM_LENGTH IS NULL)
   		  OR (cols.PropertyType = 3 AND (cols.PropertyLength < 0 AND INFO.CHARACTER_MAXIMUM_LENGTH > 0))
OPEN C2
FETCH NEXT FROM C2 INTO @column, @length, @casesensitive 
WHILE @@FETCH_STATUS = 0
    BEGIN
    --  Build and Exec the alter table statement
    SET @command = N'ALTER TABLE [inbound].[' + @table + N'] ALTER COLUMN ' + Staging.fn_ColumnDefinition(@column, 3, @casesensitive , 0, @length, 0,  NULL, 0, 0)
    EXEC (@command)

    FETCH NEXT FROM C2 INTO @column, @length, @casesensitive 

    END
CLOSE C2;
DEALLOCATE C2;

-- Set up Cursor for columns that should be dropped
-- e.g. datatype changes and new datatype is not string


DECLARE C3 CURSOR LOCAL FORWARD_ONLY READ_ONLY FOR
SELECT  N'[' + cols.TargetColumn + N']'
	FROM @columns cols
		  JOIN  [INFORMATION_SCHEMA].[COLUMNS] INFO ON (INFO.COLUMN_NAME = cols.TargetColumn AND  TABLE_NAME = @table and TABLE_SCHEMA = N'inbound')
   	WHERE   INFO.DATA_TYPE <> Staging.fn_ColumnDatatype(cols.PropertyType, 0, 0, 0) AND cols.PropertyType <> 3
OPEN C3
FETCH NEXT FROM C3 INTO @column
WHILE @@FETCH_STATUS = 0
    BEGIN
    --  Build and Exec the alter table statement
    SET @command = N'ALTER TABLE [inbound].[' + @table + N'] DROP COLUMN ' + @column 
    EXEC (@command)

    FETCH NEXT FROM C3 INTO @column

    END
CLOSE C3;
DEALLOCATE C3;

end

go
if object_id(N'Staging.MakeStagingTableForManagedType') is not null
  drop procedure Staging.MakeStagingTableForManagedType
go

if object_id(N'Staging.MakeStagingTableForRelationshipType') is not null
  drop procedure Staging.MakeStagingTableForRelationshipType
go

create procedure Staging.MakeStagingTableForRelationshipType
(
  @RelationshipTypeId uniqueidentifier
 ,@StagingDb          nvarchar(128)
 ,@MPDatasourceId     uniqueidentifier
)
as
begin
set nocount on

declare @ManagementPackId uniqueidentifier
declare @table nvarchar(128)
declare @target nvarchar(128)
declare @schema nvarchar(128)
declare @typeName nvarchar(256)
declare @ObjectGuid uniqueidentifier
declare @IndexName nvarchar(128)
declare @cmd nvarchar(max)
declare @error int
declare @columns table (RelationshipPropertyId   uniqueidentifier, 
                        RelationshipTypeId       uniqueidentifier,
                        SourceTableName          nvarchar(128),
                        ColumnName               nvarchar(128), 
                        TargetColumn             nvarchar(128),
                        PropertyType             int, 
                        IsCaseSensitive          bit, 
                        Required                 bit, 
                        PropertyLength           int, 
                        PropertyScale            int, 
                        DefaultValue             nvarchar(2000), 
                        MaxValue                 int, 
                        MinValue                 int,
                        AlreadyExists            bit default (0))
declare @existing table (ColumnName             nvarchar(128))

-- Name of the Source Table
select @ManagementPackId = ManagementPackId, @table = RelationshipTypeViewName, @typeName = RelationshipTypeName
from Staging.RelationshipType 
where RelationshipTypeId = @RelationshipTypeId

-- Name of the Target Table
--select top (1) @target = RelationshipTypeName from Staging.RelationshipType
--where  IsAbstract = 0 and RelationshipTypeId in (select RelationshipTypeId from Staging.fn_AllBaseRelationshipsAndSelf(@RelationshipTypeId))
--order by TimeAdded
set @schema = N'inbound'
set @target = @table
if @target is null return
if len(@target) > 128 set @target = @table

insert  @columns (RelationshipPropertyId, RelationshipTypeId, SourceTableName, ColumnName, TargetColumn, PropertyType,
                  IsCaseSensitive, Required, PropertyLength, PropertyScale, DefaultValue, MaxValue, MinValue) 
SELECT * FROM Staging.fn_GetStagingTableColumnsForRelationship (@RelationshipTypeId) 

set @cmd = N'create table ' + @schema + N'.[' + @target + N'] ( '
if exists (select * from INFORMATION_SCHEMA.TABLES where TABLE_NAME = @target and TABLE_SCHEMA = @schema)
  begin
    exec Staging.AlterStagingTableColumnsForRelationshipType @RelationshipTypeId
    insert @existing select COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS where TABLE_NAME = @target and TABLE_SCHEMA = @schema
    set @cmd = N'alter table ' + @schema + N'.[' + @target + N'] add '

    update a set a.AlreadyExists = 1 from @columns a join @existing b on a.TargetColumn=b.ColumnName
  end

select @cmd = @cmd + Staging.fn_ColumnDefinition(N'[' + TargetColumn + N']', PropertyType, IsCaseSensitive, 0, PropertyLength, PropertyScale, DefaultValue, MaxValue, MinValue) + N','
from @columns where AlreadyExists = 0
-- here we can add to ColumnMapping
-- use @columns to insert, use @existing to flip IsDropped bit.

if not exists (select * from INFORMATION_SCHEMA.TABLES where TABLE_NAME = @target and TABLE_SCHEMA = @schema)
  begin
    set @cmd = @cmd + N')'
  end
else
  begin
    set @cmd = substring(@cmd, 1, len(@cmd) - 1)
  end

begin tran
if exists (select 1 from @columns WHERE AlreadyExists = 0)
  begin
    exec(@cmd)

    set @error = @@error
    if @error != 0 goto error
    
    if (len(@target) + len(N'_DWTimestamp')) < 128
    	SET @IndexName =  @target + N'_DWTimestamp'
    ELSE
    	BEGIN
   	SET @ObjectGuid = cast(HASHBYTES('SHA1', @target) AS uniqueidentifier)
	SET @IndexName = SUBSTRING(@target, 1, 80) +  N'_' + REPLACE(CONVERT(nvarchar(36), @ObjectGuid), '-','_') + N'_DWTimestamp'
    	END
	
    set @cmd = N'IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE object_id = OBJECT_ID(''' + @schema + N'.[' + @target + N']'') AND name = N''' + @IndexName + N''' )
    		CREATE CLUSTERED INDEX ' + @IndexName + N' ON ' + @schema + N'.[' + @target + N'] (DWTimestamp)'
    EXEC (@cmd)
    set @error = @@error
    if @error != 0 goto error
    -- Remove DWTimestamp, BatchId, InsertedTimestamp and RejectedCount  for @columns so that we won't put it into ColumnMappings and try to query for it or provide a value
    DELETE FROM @columns WHERE SourceTableName IS NULL AND ( ColumnName IN  (N'DWTimestamp',  N'BatchId', N'InsertedTimestamp', N'RejectedCount') )
    
    --  If we are creating the staging table in the DWCMDB, then create a synonym in the SMDW database.
    EXEC Staging.p_MakeSynonymInDW @target, N'inbound'

    set @error = @@error
    if @error != 0 goto error
    
    update a set
     a.IsDropped = 1
    ,a.LastModified = getutcdate()
    from Staging.ColumnMapping a
    left join @columns b on a.PropertyId=b.RelationshipPropertyId
    where a.TypeId=@RelationshipTypeId
    and   b.RelationshipPropertyId is null

    set @error = @@error
    if @error != 0 goto error

    update a set
     a.IsDropped = 0
    ,a.LastModified = getutcdate()
    from Staging.ColumnMapping a
    join @columns b on a.PropertyId=b.RelationshipPropertyId
    where a.TypeId=@RelationshipTypeId
    and   a.IsDropped = 1

    set @error = @@error
    if @error != 0 goto error 
  end 
    
insert Staging.ColumnMapping (TypeId, PropertyId, SourceSchemaName, SourceTableName, SourceColumnName, 
                              TargetSchemaName, TargetTableName, TargetColumnName, IsDropped, IsActive, 
                              ManagementPackId, TimeAdded, LastModified)
select a.RelationshipTypeId, 
       a.RelationshipPropertyId, 
       N'dbo',
       a.SourceTableName, 
       a.ColumnName, 
       @schema,
       @target, 
       a.TargetColumn, 
       0, 
       1,
       @ManagementPackId, 
       getutcdate(), 
       getutcdate()
from Staging.fn_GetStagingTableColumnsForRelationship(@RelationshipTypeId) a
left join Staging.ColumnMapping b on b.TargetTableName=@target
      and a.RelationshipPropertyId=b.PropertyId
where a.ColumnName not in (N'BatchId', N'DWTimestamp')
and   a.RelationshipTypeId in (select RelationshipTypeId from Staging.fn_AllBaseRelationshipsAndSelf(@RelationshipTypeId))
and  (a.SourceTableName is not null or a.ColumnName in (N'DatasourceId', @typeName))
and   b.ColumnMappingId is null

set @error = @@error
if @error != 0 goto error

if exists (select 1 from Staging.TableMapping where TypeId = @RelationshipTypeId and TargetTableName <> @target and @target is not null)
	update Staging.TableMapping SET TargetTableName = @target where TypeId = @RelationshipTypeId
	
if not exists (select 1 from Staging.TableMapping where TypeId = @RelationshipTypeId)
  and not exists (select 1 from Staging.TableMapping where TargetTableName = @target)
  and @target is not null
  begin
    insert into Staging.TableMapping (TypeId, SourceTableName, SourceSchemaName, TargetTableName, TargetSchemaName, TimeAdded, LastModified)
    values (@RelationshipTypeId, @table, N'dbo', @target, @schema, getutcdate(), getutcdate())
  end

set @error = @@error
if @error != 0 goto error

commit tran

return
error:
  if XACT_STATE() != 0 and @error != 0
    begin
      rollback tran
      raiserror(@error, 16, 1)
    end
end
go

if object_id(N'Staging.AlterStagingTableColumnsForManagedType') is not null
  drop procedure Staging.AlterStagingTableColumnsForManagedType
go

create procedure Staging.AlterStagingTableColumnsForManagedType
  @ManagedTypeId     uniqueidentifier
as
begin
set nocount on

-- Name of the Source Table
declare @table nvarchar(128)
declare @column nvarchar(128)
declare @length int
declare @casesensitive bit
declare @command nvarchar(max)
select @table = ManagedTypeViewName from Staging.ManagedType where ManagedTypeId = @ManagedTypeId

declare @columns table (ManagedTypePropertyId   uniqueidentifier, 
                        ManagedTypeId           uniqueidentifier,
                        SourceTableName         nvarchar(128),
                        ColumnName              nvarchar(128), 
                        TargetColumn            nvarchar(128),
                        PropertyType            int, 
                        IsCaseSensitive         bit, 
                        Required                bit, 
                        PropertyLength          int, 
                        PropertyScale           int, 
                        DefaultValue            nvarchar(2000), 
                        MaxValue                int, 
                        MinValue                int)
insert @columns select * FROM Staging.fn_GetStagingTableColumns(@ManagedTypeId)

-- Set up Cursor over columns that should be altered
--  e.g.  strings that have length increased, and non-string datatypes
--  that are changing to string.
DECLARE C2 CURSOR LOCAL FORWARD_ONLY READ_ONLY FOR
SELECT  N'[' + cols.TargetColumn + N']', cols.PropertyLength, cols.IsCaseSensitive
    FROM @columns cols
     	      JOIN [INFORMATION_SCHEMA].[COLUMNS] INFO ON (INFO.COLUMN_NAME = cols.TargetColumn AND  TABLE_NAME = @table and TABLE_SCHEMA = N'inbound')
   WHERE   cols.PropertyType = 3 AND ((cols.PropertyLength > INFO.CHARACTER_MAXIMUM_LENGTH AND INFO.CHARACTER_MAXIMUM_LENGTH > 0) OR INFO.CHARACTER_MAXIMUM_LENGTH IS NULL)
OPEN C2
FETCH NEXT FROM C2 INTO @column, @length, @casesensitive 
WHILE @@FETCH_STATUS = 0
    BEGIN
    --  Build and Exec the alter table statement
    SET @command = N'ALTER TABLE [inbound].[' + @table + N'] ALTER COLUMN ' + Staging.fn_ColumnDefinition(@column, 3, @casesensitive , 0, @length, 0,  NULL, 0, 0)
    EXEC (@command)

    FETCH NEXT FROM C2 INTO @column, @length, @casesensitive 

    END
CLOSE C2;
DEALLOCATE C2;

-- Set up Cursor for columns that should be dropped
-- e.g. datatype changes and new datatype is not string


DECLARE C3 CURSOR LOCAL FORWARD_ONLY READ_ONLY FOR
SELECT  N'[' + cols.TargetColumn + N']'
	FROM @columns cols
		  JOIN  [INFORMATION_SCHEMA].[COLUMNS] INFO ON (INFO.COLUMN_NAME = cols.TargetColumn AND  TABLE_NAME = @table and TABLE_SCHEMA = N'inbound')
   	WHERE   INFO.DATA_TYPE <> Staging.fn_ColumnDatatype(cols.PropertyType, 0, 0, 0) AND cols.PropertyType <> 3
OPEN C3
FETCH NEXT FROM C3 INTO @column
WHILE @@FETCH_STATUS = 0
    BEGIN
    --  Build and Exec the alter table statement
    SET @command = N'ALTER TABLE [inbound].[' + @table + N'] DROP COLUMN ' + @column 
    EXEC (@command)

    FETCH NEXT FROM C3 INTO @column

    END
CLOSE C3;
DEALLOCATE C3;

end

go
if object_id(N'Staging.MakeStagingTableForManagedType') is not null
  drop procedure Staging.MakeStagingTableForManagedType
go

create procedure Staging.MakeStagingTableForManagedType
  @ManagedTypeId     uniqueidentifier
 ,@StagingDb         nvarchar(128)
 ,@MPDatasourceId    uniqueidentifier
 ,@UpdateAbstractView bit
as
begin
set nocount on

declare @ManagementPackId uniqueidentifier
declare @BaseManagedTypeId uniqueidentifier
declare @curtype uniqueidentifier
declare @relationshiptypeid uniqueidentifier
declare @hosttypeid uniqueidentifier
declare @table nvarchar(128)
declare @target nvarchar(128)
declare @schema nvarchar(128)
declare @typeName nvarchar(256)
declare @ObjectGuid uniqueidentifier
declare @IndexName nvarchar(128)
declare @cmd nvarchar(max)
declare @error int
declare @types table (typeid  uniqueidentifier)
declare @columns table (ManagedTypePropertyId   uniqueidentifier, 
                        ManagedTypeId           uniqueidentifier,
                        SourceTableName         nvarchar(128),
                        ColumnName              nvarchar(128), 
                        TargetColumn            nvarchar(128),
                        PropertyType            int, 
                        IsCaseSensitive         bit, 
                        Required                bit, 
                        PropertyLength          int, 
                        PropertyScale           int, 
                        DefaultValue            nvarchar(2000), 
                        MaxValue                int, 
                        MinValue                int,
                        AlreadyExists           bit default (0))
declare @existing table (ColumnName             nvarchar(128))

-- Name of the Source Table
select @ManagementPackId = ManagementPackId, @table = ManagedTypeViewName, @typeName = TypeName
from Staging.ManagedType where ManagedTypeId = @ManagedTypeId

-- Name of the Target Table
--select top (1) @target = TypeName from Staging.ManagedType
--where  IsAbstract = 0 and ManagedTypeId in (select ManagedTypeId from Staging.fn_AllBaseTypesAndSelf(@ManagedTypeId))
--order by TimeAdded
set @target = @table
set @schema = N'inbound'
if @target is null return
if len(@target) > 128 set @target = @table


insert @columns (ManagedTypePropertyId, ManagedTypeId, SourceTableName, ColumnName, TargetColumn, PropertyType,
                 IsCaseSensitive, Required, PropertyLength, PropertyScale, DefaultValue, MaxValue, MinValue)
select * FROM Staging.fn_GetStagingTableColumns(@ManagedTypeId)

set @cmd = N'create table ' + @schema + N'.[' + @target + N'] ( '
if exists (select * from INFORMATION_SCHEMA.TABLES where TABLE_NAME = @target and TABLE_SCHEMA = @schema)
  begin
    exec Staging.AlterStagingTableColumnsForManagedType @ManagedTypeId
    insert @existing select COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS where TABLE_NAME = @target and TABLE_SCHEMA = @schema
    set @cmd = N'alter table ' + @schema + N'.[' + @target + N'] add '

    update a set AlreadyExists = 1 from @columns a join @existing b on a.TargetColumn=b.ColumnName
  end

select @cmd = @cmd + Staging.fn_ColumnDefinition(N'[' + TargetColumn + N']', PropertyType, IsCaseSensitive, 0, PropertyLength, PropertyScale, DefaultValue, MaxValue, MinValue) + N','
from @columns where AlreadyExists = 0
-- here we can add to ColumnMapping
-- use @columns to insert, use @existing to flip IsDropped bit.

if not exists (select * from INFORMATION_SCHEMA.TABLES where TABLE_NAME = @target and TABLE_SCHEMA = @schema)
  begin
    set @cmd = @cmd + N')'
  end
else
  begin
    set @cmd = substring(@cmd, 1, len(@cmd) - 1)
  end

begin tran
if exists (select 1 from @columns WHERE AlreadyExists = 0)
  begin
    exec(@cmd)

    set @error = @@error
    if @error != 0 goto error

    if (len(@target) + len(N'_DWTimestamp')) < 128
    	SET @IndexName =  @target + N'_DWTimestamp'
    ELSE
    	BEGIN
   	SET @ObjectGuid = cast(HASHBYTES('SHA1', @target) AS uniqueidentifier)
	SET @IndexName = SUBSTRING(@target, 1, 80) +  N'_' + REPLACE(CONVERT(nvarchar(36), @ObjectGuid), '-','_') + N'_DWTimestamp'
    	END
	
   set @cmd = N'IF NOT EXISTS (SELECT 1 FROM sys.indexes WHERE object_id = OBJECT_ID(''' + @schema + N'.[' + @target + N']'') AND name = N''' + @IndexName + N''' )
    		CREATE CLUSTERED INDEX ' + @IndexName + N' ON ' + @schema + N'.[' + @target + N'] (DWTimestamp)'
    EXEC (@cmd)
    set @error = @@error
    if @error != 0 goto error
    -- Remove DWTimestamp and BatchId for @columns so that we won't put it into ColumnMappings and try to query for it or provide a value
    DELETE FROM @columns WHERE SourceTableName IS NULL AND ( ColumnName = N'DWTimestamp' OR ColumnName = N'BatchId' )
    
    --  If we are creating the staging table in the DWCMDB, then create a synonym in the SMDW database.
    EXEC Staging.p_MakeSynonymInDW @target, N'inbound'
    set @error = @@error
    if @error != 0 goto error

    -- We can't always change the view for Abstract Type right away, we should finish changing the tables for all derived types first
    IF @UpdateAbstractView = 1
    	 BEGIN
    	 -- If the Base Type was abstract then we should update or create it's Abstract Type View
    	 SELECT @BaseManagedTypeId = BaseManagedTypeId 
    			FROM Staging.ManagedType WHERE ManagedTypeId = @ManagedTypeId
    		
   	 EXEC Staging.p_MakeViewForAbstractType @BaseManagedTypeId
    	 set @error = @@error
    	 if @error != 0 goto error
   	 END

    set @error = @@error
    if @error != 0 goto error
    
    update a set
     a.IsDropped = 1
    ,a.LastModified = getutcdate()
    from Staging.ColumnMapping a
    left join @columns b on a.PropertyId=b.ManagedTypePropertyId
    where a.TypeId=@ManagedTypeId
    and   b.ManagedTypePropertyId is null

    set @error = @@error
    if @error != 0 goto error

    update a set
     a.IsDropped = 0
    ,a.LastModified = getutcdate()
    from Staging.ColumnMapping a
    join @columns b on a.PropertyId=b.ManagedTypePropertyId
    where a.TypeId=@ManagedTypeId
    and   a.IsDropped = 1

    set @error = @@error
    if @error != 0 goto error
  end
    
insert Staging.ColumnMapping (TypeId, PropertyId, SourceSchemaName, SourceTableName, SourceColumnName, 
                              TargetSchemaName, TargetTableName, TargetColumnName, IsDropped, IsActive, 
                              ManagementPackId, TimeAdded, LastModified)
select a.ManagedTypeId, 
       a.ManagedTypePropertyId, 
       N'dbo',
       a.SourceTableName, 
       a.ColumnName, 
       @schema,
       @target, 
       a.TargetColumn, 
       0, 
       1,
       @ManagementPackId, 
       getutcdate(), 
       getutcdate()
from Staging.fn_GetStagingTableColumns(@ManagedTypeId) a
left join Staging.ColumnMapping b on b.TargetTableName=@target
      and a.ManagedTypePropertyId=b.PropertyId
where a.ColumnName not in (N'BatchId', N'DWTimestamp')
and   a.ManagedTypeId in (select ManagedTypeId from Staging.fn_AllBaseTypesAndSelf(@ManagedTypeId))
and  (a.SourceTableName is not null or a.ColumnName in (N'DatasourceId', @typeName))
and   b.ColumnMappingId is null

set @error = @@error
if @error != 0 goto error

if exists (select 1 from Staging.TableMapping where TypeId = @ManagedTypeId and TargetTableName <> @target and @target is not null)
	update Staging.TableMapping SET TargetTableName = @target where TypeId = @ManagedTypeId
	
if not exists (select 1 from Staging.TableMapping where TypeId = @ManagedTypeId) 
  and not exists (select 1 from Staging.TableMapping where TargetTableName = @target)
  and @target is not null
  begin
    insert into Staging.TableMapping (TypeId, SourceTableName, SourceSchemaName, TargetTableName, TargetSchemaName, TimeAdded, LastModified)
    values (@ManagedTypeId, @table, N'dbo', @target, @schema, getutcdate(), getutcdate())
  end

set @error = @@error
if @error != 0 goto error

commit tran

return
error:
  if XACT_STATE() != 0 and @error != 0
    begin
      rollback tran
      raiserror(@error, 16, 1)
    end
end
go

if object_id(N'Staging.StagingTableCoordinator') is not null
  drop procedure Staging.StagingTableCoordinator
go

create procedure Staging.StagingTableCoordinator
  @ManagedTypeId      uniqueidentifier
 ,@RelationshipTypeId uniqueidentifier
 ,@StagingDb          nvarchar(128)
 ,@MPDatasourceId     uniqueidentifier
as
begin
declare @error int
       ,@table nvarchar(128)
       ,@IsSingleton bit
       ,@IsAbstract bit
       ,@IsHosted bit
       ,@PropertyCount int
       ,@DerivedTypeId uniqueidentifier
       ,@NonAbstractBaseTypeId uniqueidentifier

  if (@ManagedTypeId is null and @RelationshipTypeId is null) or
     (@ManagedTypeId is not null and @RelationshipTypeId is not null)
    begin
      raiserror(778000000, 16, 1)
      set @error = @@error
      goto cleanup
    end

  -- first do a simple count of properties, then -if 0- count against Staging.fn_AllBaseTypesAndSelf
  select @table = ManagedTypeViewName, @IsSingleton = IsSingleton, @IsAbstract = IsAbstract, @IsHosted = IsHosted
  from   Staging.ManagedType where ManagedTypeId = @ManagedTypeId

 -- If we are seeing AbstractType for first time, make its view now.  If we have seen it before, we should wait until
 -- later in this sproc, when we have finished processing the derived types.
  if @IsAbstract = 1 AND NOT exists (select * from Staging.ColumnMapping where TypeId = @ManagedTypeId)
     	      EXEC Staging.p_MakeViewForAbstractType @ManagedTypeId

  select @PropertyCount = count(*) from Staging.ManagedTypeProperty where ManagedTypeId = @ManagedTypeId
  if @IsSingleton = 1 and @PropertyCount = 0
    begin
      select @PropertyCount = COUNT(1) from Staging.ManagedTypeProperty 
      where ManagedTypeId in (select ManagedTypeId from Staging.fn_AllBaseTypesAndSelf(@ManagedTypeId))
      and   ManagedTypeId != (select ManagedTypeId from Staging.ManagedType where TypeName = N'System.Entity')
    end

  -- replaces query of INFORMATION_SCHEMA because source table name may not match target table name.
  if exists (select * from Staging.ColumnMapping where TypeId = @ManagedTypeId)
    begin
      if (@IsSingleton = 1 and @PropertyCount > 0 and 
        not exists (select 1 from Staging.[ManagedTypeProperty] 
                    where ManagedTypeId = @ManagedTypeId))
        begin
          print N'Singleton with properties not from self'
          -- exec table for type  [p_CreateTableForType]
          -- exec against non-abstract base type [p_CreateViewForType]
          select @NonAbstractBaseTypeId = Staging.fn_NonAbstractBaseType(@ManagedTypeId)
          exec Staging.MakeStagingTableForManagedType @NonAbstractBaseTypeId, @StagingDb, @MPDatasourceId, 1
        end
      else
        begin
          if @IsAbstract = 1
            begin
              -- get list of derived types
              declare @types table (typeid  uniqueidentifier)
              insert @types
              select DerivedManagedTypeId from Staging.fn_DerivedManagedTypes(@ManagedTypeId)
              -- iterate list, exec alter for each derived type. [p_AlterTableForType]
              print N'Abstract Type being updated'
              while exists (select 1 from @types)
                begin
                  select top 1 @DerivedTypeId = typeid from @types
                  exec Staging.MakeStagingTableForManagedType @DerivedTypeId, @StagingDb, @MPDatasourceId, 0
                  delete @types where typeid = @DerivedTypeId
                end
              if @PropertyCount > 0
                begin
                  -- if it has properties and a nonabstract base, exec for base. [p_CreateViewForType]
                  print N'Abstract has properties and so a nonabstract base type'
                  select @NonAbstractBaseTypeId = Staging.fn_NonAbstractBaseType(@ManagedTypeId)
                  exec Staging.MakeStagingTableForManagedType @NonAbstractBaseTypeId, @StagingDb, @MPDatasourceId, 0
                end
    	      EXEC Staging.p_MakeViewForAbstractType @ManagedTypeId
            end
          else
            begin
              -- exec alter for the type [p_AlterTableForType]
              print N'Non abstract Type being updated'
              exec Staging.MakeStagingTableForManagedType @ManagedTypeId, @StagingDb, @MPDatasourceId, 1
            end
          -- add foreign keys [p_AddFKToTable]
        end

      if @IsAbstract = 0
        begin
          if @IsSingleton = 1 and @PropertyCount = 0
            begin
              -- exec for singleton [p_CreateViewForSingletonType]
              print N'Singleton without properties'
              exec Staging.MakeStagingTableForManagedType @ManagedTypeId, @StagingDb, @MPDatasourceId, 1
              update Staging.[ManagedType] set ManagedTypeTableName = null 
              where ManagedTypeId = @ManagedTypeId
            end
          else
            begin
              -- exec for nonabstract base. [p_CreateViewForType]
              print N'Something with properties that is not abstract'
              select @NonAbstractBaseTypeId = Staging.fn_NonAbstractBaseType(@ManagedTypeId)
              exec Staging.MakeStagingTableForManagedType @NonAbstractBaseTypeId, @StagingDb, @MPDatasourceId, 1
            end
        end
    end
  else if @table is not null
    begin
      -- metadata work
      if @IsAbstract = 0 and (@IsSingleton = 0 or @PropertyCount > 0)
        begin
          -- exec table for type [p_CreateTableForType]
          -- exec for nonabstract base. [p_CreateViewForType]
          print N'Not abstract and has properties or is not singleton'
          exec Staging.MakeStagingTableForManagedType @ManagedTypeId, @StagingDb, @MPDatasourceId, 1
          -- add fks.
        end
      if @IsSingleton = 1 and @PropertyCount = 0
        begin
          print N'Singleton without properties'
          update Staging.[ManagedType] set ManagedTypeTableName = null 
          where ManagedTypeId = @ManagedTypeId
        end
      if @IsSingleton = 1 and @IsHosted = 0
        begin
          -- register the singleton [p_ManagedEntityInsert]
          print N'Unhosted Singleton'
          if @PropertyCount = 0
            begin
              -- exec for singleton type [p_CreateViewForSingletonType]
              print N'Unhosted Singleton needs view.'
              exec Staging.MakeStagingTableForManagedType @ManagedTypeId, @StagingDb, @MPDatasourceId, 1
            end
        end
    end

  -- For Abstract Relationship Types, create a view
  if exists (select 1 from Staging.RelationshipType where RelationshipTypeId = @RelationshipTypeId   and   IsAbstract = 1)
  	exec Staging.p_MakeViewForAbstractRelationshipType  @RelationshipTypeId 
  	
  set @table = null
  select @table = RelationshipTypeViewName
  from   Staging.RelationshipType
  where RelationshipTypeId = @RelationshipTypeId
  and   IsAbstract = 0
  if @table is not null
    begin
      print N'There a table named for the relationship'
      -- create relationship table
      exec Staging.MakeStagingTableForRelationshipType @RelationshipTypeId, @StagingDb, @MPDatasourceId
    end

cleanup:

return
end
go

if object_id(N'Staging.AddClass') is not null
  drop procedure Staging.AddClass
go

create procedure Staging.AddClass
  @ManagementPackId uniqueidentifier
 ,@MPName           nvarchar(256)
 ,@MPKeyToken       varchar(32)
 ,@MPDatasourceId   uniqueidentifier
 ,@ViewName         nvarchar(128)
 ,@TableName        nvarchar(128)
 ,@StagingDb        nvarchar(128)
 ,@Xml              xml
 ,@Delete           bit                = 0
as
begin
DECLARE @IsSealed bit
DECLARE @IsAbstract bit
DECLARE @DocHandle int
DECLARE @SaveError int
DECLARE @RowCount  int
DECLARE @ManagedTypeTableName nvarchar(128)
DECLARE @ManagedTypeViewName  nvarchar(128)
DECLARE @BaseManagedTypeId uniqueidentifier
DECLARE @ManagedTypeId uniqueidentifier
DECLARE @XmlText nvarchar(max)
DECLARE @command nvarchar(max)

SET @XmlText = CONVERT(nvarchar(max), @Xml)
EXEC sp_xml_preparedocument  @DocHandle OUTPUT, @XmlText ,
    N'<ROOT xmlns:pb="http://schemas.microsoft.com/mom/reporting/2007/ReportParameterSettings"/>'

BEGIN TRAN

IF LEN(@MPKeyToken) > 0 SET @IsSealed = 1 ELSE SET @IsSealed = 0

-- SHOULD TEST TO VERIFY @Id = @ManagedTypeId
SELECT @ManagedTypeId = Staging.fn_MPObjectId(@MPName, @MPKeyToken, ID)
      ,@TableName = coalesce(@TableName, Staging.fn_DeterministicObjectName(N'MT_', ID))
      ,@ViewName  = coalesce(@ViewName, Staging.fn_DeterministicObjectName(N'MTV_', ID))
FROM OPENXML(@DocHandle, N'/ClassType', 3) 
WITH (   ID  nvarchar(256))
SET @SaveError = @@Error
IF @SaveError <> 0 GOTO Cleanup

DELETE Staging.ManagedTypeProperty WHERE ManagedTypeId = @ManagedTypeId
SET @SaveError = @@Error
IF @SaveError <> 0 GOTO Cleanup

IF @Delete = 1
BEGIN
  IF EXISTS (SELECT * FROM Staging.ExtractionType WHERE TypeId = @ManagedTypeId and IsDeleted = 0)
  BEGIN
    SET @SaveError = 778000003 /* custom error required */
    IF @SaveError <> 0 GOTO Cleanup
  END

  -- Before we delete the row from Staging.ManagedType, get the BaseManagedTypeId so that
  -- we can find the abstract view which needs to be redefined.
  SELECT @BaseManagedTypeId = BaseManagedTypeId FROM Staging.ManagedType WHERE ManagedTypeId = @ManagedTypeId
  DELETE Staging.ManagedType WHERE ManagedTypeId = @ManagedTypeId
  SET @SaveError = @@Error
  IF @SaveError <> 0 GOTO Cleanup

  DELETE Staging.ColumnMapping WHERE TypeId = @ManagedTypeId
  SET @SaveError = @@Error
  IF @SaveError <> 0 GOTO Cleanup

 -- DROP TABLE STAGING TABLE
  IF EXISTS (SELECT 1 FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = 'inbound' AND TABLE_NAME = @ViewName) AND
  	NOT EXISTS (SELECT 1 FROM Staging.ManagedType WHERE ManagedTypeViewName = @ViewName)
  	BEGIN
  	-- Re-compute the Abstract view that would pull data from this table
  	SELECT @IsAbstract = IsAbstract FROM Staging.ManagedType WHERE ManagedTypeId = @BaseManagedTypeId
  	WHILE (@IsAbstract = 0)
  		BEGIN
  		SELECT @BaseManagedTypeId = BaseManagedTypeId FROM Staging.ManagedType WHERE ManagedTypeId = @BaseManagedTypeId
  		SELECT @IsAbstract = IsAbstract FROM Staging.ManagedType WHERE ManagedTypeId = @BaseManagedTypeId
  		END
   	EXEC Staging.p_MakeViewForAbstractType @BaseManagedTypeId

  	IF EXISTS (SELECT 1 FROM INFORMATION_SCHEMA.VIEWS WHERE TABLE_SCHEMA = 'inbound' AND TABLE_NAME = @ViewName) 
  		SET  @command = N'DROP VIEW inbound.' + @ViewName ELSE SET @command = N'DROP TABLE inbound.' + @ViewName
	EXEC (@command)
  	END
  COMMIT TRAN
  GOTO Cleanup
END

-- Load ManagedTypes, ManagedTypeProperties into staging tables
IF EXISTS (SELECT * FROM Staging.ManagedType WHERE ManagedTypeId = @ManagedTypeId)
BEGIN
  UPDATE a SET
   a.IsHosted = COALESCE(b.Hosted, 0)
  ,a.IsAbstract = COALESCE(b.Abstract, 0)
  ,a.BaseManagedTypeId = Staging.fn_MPReferencedObjectId(@MPName, @MPKeyToken, Base)
  ,a.Accessibility = Staging.fn_MapAccessibilityEnum(b.Accessibility)
  ,a.IsSingleton = COALESCE(b.Singleton, 0)
  ,a.IsSealed = @IsSealed
  ,a.ManagedTypeTableName = @TableName
  ,a.ManagedTypeViewName = @ViewName
  ,a.LastModified = getutcdate()
  FROM Staging.ManagedType a,
   OPENXML(@DocHandle, N'/ClassType', 3) 
   WITH ( ID  nvarchar(256),
          Base nvarchar(512),
          Accessibility nvarchar(100),
          Abstract  bit,
          Hosted    bit,
          Singleton bit) b
   WHERE a.ManagedTypeId = @ManagedTypeId
   SET @SaveError = @@Error
   IF @SaveError <> 0 GOTO Cleanup
END
ELSE
BEGIN
   INSERT INTO Staging.[ManagedType] (ManagedTypeId, TypeName, BaseManagedTypeId, ManagementPackId, IsHosted, IsAbstract, Accessibility,
                IsSingleton, IsSealed, ManagedTypeTableName, ManagedTypeViewName)
   SELECT Staging.fn_MPObjectId(@MPName, @MPKeyToken, ID), ID, Staging.fn_MPReferencedObjectId(@MPName, @MPKeyToken, Base),
        @ManagementPackId, COALESCE(Hosted, 0),  COALESCE(Abstract, 0), Staging.fn_MapAccessibilityEnum(Accessibility), COALESCE(Singleton, 0), @IsSealed,
        @TableName,
        @ViewName
   FROM OPENXML(@DocHandle, N'/ClassType', 3) 
   WITH ( ID  nvarchar(256),
          Base nvarchar(512),
          Accessibility nvarchar(100),
          Abstract  bit,
          Hosted    bit,
          Singleton bit)
   SET @SaveError = @@Error
   IF @SaveError <> 0 GOTO Cleanup
END

UPDATE Staging.[ManagedType] SET ManagedTypeTableName = NULL, ManagedTypeViewName = NULL
WHERE IsAbstract = 1  AND ManagedTypeId = @ManagedTypeId
    
INSERT INTO Staging.[ManagedTypeProperty] (ManagedTypePropertyId, ManagedTypeId, ManagedTypePropertyName,
                          IsPrimaryKey, IsCaseSensitive, PropertyType, PropertyLength, PropertyMinLength, PropertyScale, ColumnName, MinValue, MaxValue, RegEx, EnumTypeId, Optimized, Required, DefaultValue,
                          ManagementPackId, InternalOrder)
    SELECT Staging.fn_MPSubObjectId(@MPName, @MPKeyToken, ClassName, PropName), Staging.fn_MPObjectId(@MPName, @MPKeyToken, ClassName), PropName, 
     COALESCE(IsKey, 0), COALESCE(CaseSensitive, 0), Staging.fn_MapDatatypeEnum(Type), COALESCE(COALESCE(PropertyLength, PropertyLengthAlt), Staging.fn_MapDatatypeDefaultLength(Type)), COALESCE(PropertyMinLength, 0), COALESCE(PropertyScale, 0),
         Staging.fn_DeterministicColumnName(ClassName, PropName), MinValue, MaxValue, RegEx, 
         Staging.fn_MPReferencedObjectId(@MPName, @MPKeyToken, EnumType), Optimized, Required, DefaultValue,
         @ManagementPackId, mp_id
     FROM OPENXML (@DocHandle, N'/ClassType/Property', 3) 
 WITH (   PropName  nvarchar(256) N'@ID' ,
          Type nvarchar(256) ,
          PropertyLength int N'@Length',
          PropertyLengthAlt int N'@MaxLength',
          PropertyMinLength int N'@MinLength',
          PropertyScale int N'@Scale',
          MinValue int,
          MaxValue    int,
          RegEx  nvarchar(256),
          EnumType  nvarchar(512),
          DefaultValue nvarchar(2000),
          Required bit,
          Optimized   bit,
          ClassName varchar(256) N'../@ID',
          mp_id int N'@mp:id',
          IsKey bit N'@Key',
          CaseSensitive bit) 
SET @SaveError = @@Error
IF @SaveError <> 0 GOTO Cleanup

-- Now set the ColumnOrder using the InternalOrder
UPDATE Staging.[ManagedTypeProperty]  SET ColumnOrder =
    (SELECT 1 + COUNT(1) from Staging.ManagedTypeProperty MTP2 where MTP2.ManagedTypeId = MTP.ManagedTypeId AND
    MTP2.InternalOrder < MTP.InternalOrder)
FROM Staging.ManagedTypeProperty MTP WHERE  ManagementPackId = @ManagementPackId
SET @SaveError = @@Error
IF @SaveError <> 0 GOTO Cleanup

--IF @IsSealed = 1
--    BEGIN
--    -- For a few special internal types use PropertyName as ColumnName without appending the Guid
--      UPDATE Staging.[ManagedTypeProperty] SET ColumnName = ManagedTypePropertyName WHERE ManagedTypeId = (select ManagedTypeId from Staging.ManagedType where TypeName = 'Microsoft.Windows.Computer')
--      SET @SaveError = @@Error
--      IF @SaveError <> 0 GOTO Cleanup
--      UPDATE Staging.[ManagedTypeProperty] SET ColumnName = ManagedTypePropertyName WHERE ManagedTypeId = (select ManagedTypeId from Staging.ManagedType where TypeName = 'Microsoft.SystemCenter.HealthService')
--      SET @SaveError = @@Error
--      IF @SaveError <> 0 GOTO Cleanup
--      UPDATE Staging.[ManagedTypeProperty] SET ColumnName = ManagedTypePropertyName WHERE ManagedTypeId = (select ManagedTypeId from Staging.ManagedType where TypeName = 'Microsoft.SystemCenter.Connector')
--      SET @SaveError = @@Error
--      IF @SaveError <> 0 GOTO Cleanup
--      UPDATE Staging.[ManagedTypeProperty] SET ColumnName = ManagedTypePropertyName WHERE ManagedTypeId = (select ManagedTypeId from Staging.ManagedType where TypeName = 'Microsoft.SystemCenter.CMAEMMapperType')
--      SET @SaveError = @@Error
--      IF @SaveError <> 0 GOTO Cleanup
--      UPDATE Staging.[ManagedTypeProperty] SET ColumnName = ManagedTypePropertyName WHERE ManagedTypeId = (select ManagedTypeId from Staging.ManagedType where TypeName = 'Microsoft.SystemCenter.CMAEMWatsonBucket')
--      SET @SaveError = @@Error
--      IF @SaveError <> 0 GOTO Cleanup
--      UPDATE Staging.[ManagedTypeProperty] SET ColumnName = ManagedTypePropertyName WHERE ManagedTypeId = (select ManagedTypeId from Staging.ManagedType where TypeName = 'Microsoft.SystemCenter.CMAEMMonitorOverride')
--      SET @SaveError = @@Error
--      IF @SaveError <> 0 GOTO Cleanup
--      UPDATE Staging.[ManagedTypeProperty] SET ColumnName = ManagedTypePropertyName WHERE ManagedTypeId = (select ManagedTypeId from Staging.ManagedType where TypeName = 'Microsoft.SystemCenter.CMAEMApplication')
--      SET @SaveError = @@Error
--      IF @SaveError <> 0 GOTO Cleanup
--    END

  SELECT  @ManagedTypeTableName = ManagedTypeTableName FROM Staging.[ManagedType] WHERE ManagedTypeId = @ManagedTypeId
  --WHILE EXISTS (SELECT 1 FROM Staging.[ManagedType] WHERE ManagedTypeTableName = @ManagedTypeTableName AND ManagedTypeId != @ManagedTypeId) OR
  --    EXISTS (SELECT 1 FROM Staging.[RelationshipType] WHERE RelationshipTypeTableName = @ManagedTypeTableName)
  --   BEGIN
  --     SET @ManagedTypeTableName = Staging.fn_NextName(@ManagedTypeTableName)
  --   END

  SELECT @ManagedTypeViewName = ManagedTypeViewName, @BaseManagedTypeId = BaseManagedTypeId FROM Staging.[ManagedType] WHERE ManagedTypeId = @ManagedTypeId
  IF EXISTS (SELECT 1 FROM Staging.[ManagedType] WHERE ManagedTypeId = @BaseManagedTypeId AND IsAbstract = 0)
    BEGIN
      SELECT @ManagedTypeViewName = ManagedTypeViewName FROM Staging.ManagedType
      WHERE ManagedTypeId = @BaseManagedTypeId AND IsAbstract = 0
    END
  --ELSE 
  --  BEGIN
  --    WHILE EXISTS (SELECT 1 FROM Staging.[ManagedType] WHERE ManagedTypeViewName = @ManagedTypeViewName AND ManagedTypeId != @ManagedTypeId) OR
  --          EXISTS (SELECT 1 FROM Staging.[RelationshipType] WHERE RelationshipTypeViewName = @ManagedTypeViewName)
  --      BEGIN
  --        SET @ManagedTypeViewName = Staging.fn_NextName(@ManagedTypeViewName)
  --      END
  --   END
  UPDATE Staging.ManagedType SET
    ManagedTypeTableName = @ManagedTypeTableName
   ,ManagedTypeViewName  = @ManagedTypeViewName
  WHERE ManagedTypeId = @ManagedTypeId
  SET @SaveError = @@Error
  IF @SaveError <> 0 GOTO Cleanup

if @ManagedTypeId is not null
  exec Staging.StagingTableCoordinator @ManagedTypeId, NULL, @StagingDb, @MPDatasourceId
SET @SaveError = @@Error
IF @SaveError <> 0 GOTO Cleanup

SELECT @RowCount = count(*) FROM Staging.ManagedType where ManagedTypeId = @ManagedTypeId
IF (@RowCount = 0 and @Delete = 0)
BEGIN
  SET @SaveError = 778000003
  GOTO Cleanup
END

COMMIT TRAN

Cleanup:
 -- cleanup  
IF (@DocHandle IS NOT NULL)
   EXEC sp_xml_removedocument @DocHandle
   
IF (@SaveError != 0)
    BEGIN
    IF (XACT_STATE() != 0) BEGIN ROLLBACK TRAN END
    RAISERROR(@SaveError, 16, 1)
    RETURN 1
    END
RETURN 0

end
go

if object_id(N'Staging.AddRelationship') is not null
  drop procedure Staging.AddRelationship
go

create procedure Staging.AddRelationship
  @ManagementPackId uniqueidentifier
 ,@MPName           nvarchar(256)
 ,@MPKeyToken       varchar(32)
 ,@MPDatasourceId   uniqueidentifier
 ,@ViewName         nvarchar(128)
 ,@TableName        nvarchar(128)
 ,@StagingDb        nvarchar(128)
 ,@Xml              xml
 ,@Delete           bit                   = 0
as
begin
DECLARE @IsSealed bit
DECLARE @DocHandle int
DECLARE @SaveError int
DECLARE @RowCount  int
DECLARE @RelationshipTableName nvarchar(128)
DECLARE @RelationshipViewName nvarchar(128)
DECLARE @RelationshipTypeId uniqueidentifier
DECLARE @XmlText nvarchar(max)

SET @XmlText = CONVERT(nvarchar(max), @Xml)
EXEC sp_xml_preparedocument  @DocHandle OUTPUT, @XmlText ,
    N'<ROOT xmlns:pb="http://schemas.microsoft.com/mom/reporting/2007/ReportParameterSettings"/>'

BEGIN TRAN

IF LEN(@MPKeyToken) > 0 SET @IsSealed = 1 ELSE SET @IsSealed = 0

-- SHOULD TEST TO VERIFY @Id = @RelationshipTypeId
SELECT @RelationshipTypeId = Staging.fn_MPObjectId(@MPName, @MPKeyToken, RelationshipTypeName)
FROM OPENXML (@DocHandle,  N'/RelationshipType', 3) 
WITH ( RelationshipTypeName nvarchar(256) N'@ID')
SET @SaveError = @@Error
IF @SaveError <> 0 GOTO Cleanup

DELETE Staging.RelationshipTypeProperty WHERE RelationshipTypeId = @RelationshipTypeId
SET @SaveError = @@Error
IF @SaveError <> 0 GOTO Cleanup

IF @Delete = 1
BEGIN
  IF EXISTS (SELECT * FROM Staging.ExtractionType WHERE TypeId = @RelationshipTypeId and IsDeleted = 0)
  BEGIN
    SET @SaveError = 778000003 /* custom error required */
    IF @SaveError <> 0 GOTO Cleanup
  END

  DELETE Staging.RelationshipType WHERE RelationshipTypeId = @RelationshipTypeId
  SET @SaveError = @@Error
  IF @SaveError <> 0 GOTO Cleanup

  DELETE Staging.ColumnMapping WHERE TypeId = @RelationshipTypeId
  SET @SaveError = @@Error
  IF @SaveError <> 0 GOTO Cleanup

  COMMIT TRAN
  GOTO Cleanup
END

-- Load RelationshipTypes, RelationshipTypeProperties into staging tables
IF EXISTS (SELECT * FROM Staging.RelationshipType WHERE RelationshipTypeId = @RelationshipTypeId)
BEGIN
  UPDATE a SET
   a.SourceManagedTypeId = Staging.fn_MPReferencedObjectId(@MPName, @MPKeyToken, b.Source)
  ,a.SourceName = N'Source'
  ,a.TargetManagedTypeId = Staging.fn_MPReferencedObjectId(@MPName, @MPKeyToken, b.Target)
  ,a.TargetName = N'Target'
  ,a.IsAbstract = COALESCE(b.Abstract,0)
  ,a.Accessibility = Staging.fn_MapAccessibilityEnum(b.Accessibility)
  ,a.IsSealed = @IsSealed
  ,a.IsCycleOk = (CASE WHEN b.RelationshipTypeName LIKE N'%Reference%' OR b.BaseRelationshipType LIKE N'%Reference%' THEN 1 ELSE 0 END)
  ,a.RelationshipTypeTableName = @TableName
  ,a.RelationshipTypeViewName = @ViewName
  ,a.LastModified = getutcdate()
    FROM Staging.RelationshipType a,
    OPENXML (@DocHandle,  N'/RelationshipType', 3) 
    WITH ( RelationshipTypeName nvarchar(256) N'@ID',
            BaseRelationshipType nvarchar(512) N'@Base',
            Source nvarchar(512),
            Target nvarchar(512),
             Accessibility nvarchar(100),
            Abstract bit) b
   WHERE b.Source <> N'' AND a.RelationshipTypeId = @RelationshipTypeId
  SET @SaveError = @@Error
  IF @SaveError <> 0 GOTO Cleanup

  UPDATE a SET
   a.SourceManagedTypeId = Staging.fn_MPReferencedObjectId(@MPName, @MPKeyToken, b.Source)
  ,a.SourceName = b.SourceName
  ,a.SourceId = Staging.fn_MPSubObjectId(@MPName, @MPKeyToken, b.RelationshipTypeName, b.SourceName)
  ,a.SourceMaxCardinality = b.SourceMaxCardinality
  ,a.SourceMinCardinality = b.SourceMinCardinality
  ,a.TargetManagedTypeId = Staging.fn_MPReferencedObjectId(@MPName, @MPKeyToken, c.Target)
  ,a.TargetName = c.TargetName
  ,a.TargetId = Staging.fn_MPSubObjectId(@MPName, @MPKeyToken, b.RelationshipTypeName, c.TargetName)
  ,a.TargetMaxCardinality = c.TargetMaxCardinality
  ,a.TargetMinCardinality = c.TargetMinCardinality
  ,a.IsAbstract = COALESCE(b.Abstract,0)
  ,a.Accessibility = Staging.fn_MapAccessibilityEnum(b.Accessibility)
  ,a.IsSealed = @IsSealed
  ,a.IsCycleOk = (CASE WHEN b.RelationshipTypeName LIKE N'%Reference%' OR b.BaseRelationshipType LIKE N'%Reference%' THEN 1 ELSE 0 END)
  ,a.RelationshipTypeTableName = @TableName
  ,a.RelationshipTypeViewName = @ViewName
  ,a.LastModified = getutcdate()
  FROM Staging.RelationshipType a,
  OPENXML (@DocHandle,  N'/RelationshipType/Source', 3) 
    WITH ( RelationshipTypeName nvarchar(256) N'../@ID',
            BaseRelationshipType nvarchar(512) N'../@Base',
            SourceName nvarchar(512) N'@ID',
            Source nvarchar(512) N'@Type',
            SourceMaxCardinality int N'@MaxCardinality',
            SourceMinCardinality int N'@MinCardinality',
            Accessibility nvarchar(100) N'../@Accessibility',
            Abstract bit N'../@Abstract') b
    JOIN OPENXML (@DocHandle,  N'/RelationshipType/Target', 3) 
    WITH ( TargetRelationshipTypeName nvarchar(256) N'../@ID',
            TargetName nvarchar(512) N'@ID',
            TargetMaxCardinality int N'@MaxCardinality',
            TargetMinCardinality int N'@MinCardinality',
            Target nvarchar(512) N'@Type') c
   ON c.TargetRelationshipTypeName  = b.RelationshipTypeName
  WHERE a.RelationshipTypeId = @RelationshipTypeId
  SET @SaveError = @@Error
  IF @SaveError <> 0 GOTO Cleanup
END
ELSE
BEGIN
   INSERT INTO Staging.[RelationshipType]   (RelationshipTypeId, RelationshipTypeName, BaseRelationshipTypeId,
                                          SourceManagedTypeId, SourceName, TargetManagedTypeId, TargetName, ManagementPackId,
                                          IsAbstract, Accessibility, IsSealed, IsCycleOk,
                                          RelationshipTypeTableName, RelationshipTypeViewName)
    SELECT Staging.fn_MPObjectId(@MPName, @MPKeyToken, RelationshipTypeName), RelationshipTypeName,
            Staging.fn_MPReferencedObjectId(@MPName, @MPKeyToken, BaseRelationshipType), 
            Staging.fn_MPReferencedObjectId(@MPName, @MPKeyToken, Source), N'Source',
            Staging.fn_MPReferencedObjectId(@MPName, @MPKeyToken, Target), N'Target',
            @ManagementPackId, COALESCE(Abstract,0), Staging.fn_MapAccessibilityEnum(Accessibility), @IsSealed, (CASE WHEN RelationshipTypeName LIKE N'%Reference%' OR
                BaseRelationshipType LIKE N'%Reference%' THEN 1 ELSE 0 END), @TableName, @ViewName
    FROM OPENXML (@DocHandle,  N'/RelationshipType', 3) 
    WITH ( RelationshipTypeName nvarchar(256) N'@ID',
            BaseRelationshipType nvarchar(512) N'@Base',
            Source nvarchar(512),
            Target nvarchar(512),
             Accessibility nvarchar(100),
            Abstract bit)
   WHERE Source <> ''
  SET @SaveError = @@Error
  IF @SaveError <> 0 GOTO Cleanup

  INSERT INTO Staging.[RelationshipType]    (RelationshipTypeId, RelationshipTypeName, BaseRelationshipTypeId,
                                          SourceManagedTypeId, SourceName, SourceId, SourceMaxCardinality, SourceMinCardinality,
                                          TargetManagedTypeId, TargetName, TargetId, TargetMaxCardinality, TargetMinCardinality, ManagementPackId,
                                          IsAbstract, Accessibility, IsSealed, IsCycleOk,
                                          RelationshipTypeTableName, RelationshipTypeViewName)
    SELECT Staging.fn_MPObjectId(@MPName, @MPKeyToken, RelationshipTypeName), RelationshipTypeName,
            Staging.fn_MPReferencedObjectId(@MPName, @MPKeyToken, BaseRelationshipType), 
            Staging.fn_MPReferencedObjectId(@MPName, @MPKeyToken, Source), SourceName,  Staging.fn_MPSubObjectId(@MPName, @MPKeyToken, RelationshipTypeName, SourceName),
            SourceMaxCardinality, SourceMinCardinality,
            Staging.fn_MPReferencedObjectId(@MPName, @MPKeyToken, Target), TargetName, Staging.fn_MPSubObjectId(@MPName, @MPKeyToken, RelationshipTypeName, TargetName),
            TargetMaxCardinality, TargetMinCardinality,
            @ManagementPackId, COALESCE(Abstract,0), Staging.fn_MapAccessibilityEnum(Accessibility), @IsSealed, (CASE WHEN RelationshipTypeName LIKE N'%Reference%' OR
                BaseRelationshipType LIKE N'%Reference%' THEN 1 ELSE 0 END), @TableName, @ViewName
    FROM OPENXML (@DocHandle,  N'/RelationshipType/Source', 3) 
    WITH ( RelationshipTypeName nvarchar(256) N'../@ID',
            BaseRelationshipType nvarchar(512) N'../@Base',
            SourceName nvarchar(512) N'@ID',
            Source nvarchar(512) N'@Type',
            SourceMaxCardinality int N'@MaxCardinality',
            SourceMinCardinality int N'@MinCardinality',
            Accessibility nvarchar(100) N'../@Accessibility',
            Abstract bit N'../@Abstract')
    JOIN OPENXML (@DocHandle,  N'/RelationshipType/Target', 3) 
    WITH ( TargetRelationshipTypeName nvarchar(256) N'../@ID',
            TargetName nvarchar(512) N'@ID',
            TargetMaxCardinality int N'@MaxCardinality',
            TargetMinCardinality int N'@MinCardinality',
            Target nvarchar(512) N'@Type')
   ON TargetRelationshipTypeName  = RelationshipTypeName
  SET @SaveError = @@Error
  IF @SaveError <> 0 GOTO Cleanup
END

INSERT INTO Staging.[RelationshipTypeProperty]     (RelationshipPropertyId, RelationshipTypeId, RelationshipPropertyName, IsCaseSensitive,
                            PropertyType, PropertyLength, ColumnName, MinValue, MaxValue, RegEx, EnumTypeId, Optimized, Required, DefaultValue, ManagementPackId)
    SELECT Staging.fn_MPSubObjectId(@MPName, @MPKeyToken, RelTypeName, PropName),
            Staging.fn_MPObjectId(@MPName, @MPKeyToken, RelTypeName), PropName, 
             COALESCE(CaseSensitive, 0), Staging.fn_MapDatatypeEnum(Type), COALESCE(COALESCE(PropertyLength, PropertyLengthAlt), Staging.fn_MapDatatypeDefaultLength(Type)), Staging.fn_DeterministicColumnName(RelTypeName, PropName), 
             MinValue, MaxValue, RegEx, Staging.fn_MPReferencedObjectId(@MPName, @MPKeyToken, EnumType), 
             Optimized, Required, DefaultValue, @ManagementPackId
     FROM OPENXML (@DocHandle, N'/RelationshipType/Property', 3) 
 WITH (   PropName  nvarchar(256) N'@ID' ,
          Type nvarchar(256) ,
          PropertyLength int N'@Length',
          PropertyLengthAlt int N'@MaxLength',
          RelTypeName varchar(256) N'../@ID',
          MinValue int,
          MaxValue    int,
          RegEx  nvarchar(256),
          EnumType  nvarchar(512),
          DefaultValue nvarchar(2000),
          Required bit,
          Optimized   bit,
          CaseSensitive bit) 
SET @SaveError = @@Error
IF @SaveError <> 0 GOTO Cleanup

-- Update Table and View
SELECT @RelationshipTableName = RelationshipTypeTableName, @RelationshipViewName = RelationshipTypeViewName
FROM Staging.[RelationshipType] WHERE RelationshipTypeId = @RelationshipTypeId
  
--WHILE EXISTS (SELECT 1 FROM Staging.[ManagedType] WHERE ManagedTypeTableName = @RelationshipTableName) OR
--      EXISTS (SELECT 1 FROM Staging.[RelationshipType] WHERE RelationshipTypeTableName = @RelationshipTableName AND
--              RelationshipTypeId <> @RelationshipTypeId)
--  BEGIN
--    SET @RelationshipTableName = Staging.fn_NextName(@RelationshipTableName)
--  END
       
--WHILE EXISTS (SELECT 1 FROM Staging.[ManagedType] WHERE ManagedTypeViewName = @RelationshipViewName) OR
--      EXISTS (SELECT 1 FROM Staging.[RelationshipType] WHERE RelationshipTypeViewName = @RelationshipViewName AND
--              RelationshipTypeId <> @RelationshipTypeId)
--  BEGIN
--    SET @RelationshipViewName = Staging.fn_NextName(@RelationshipViewName)
--  END

if @RelationshipTypeId is not null
  exec Staging.StagingTableCoordinator NULL, @RelationshipTypeId, @StagingDb, @MPDatasourceId
SET @SaveError = @@Error
IF @SaveError <> 0 GOTO Cleanup

SELECT @RowCount = count(*) FROM Staging.RelationshipType where RelationshipTypeId = @RelationshipTypeId
IF (@RowCount = 0 and @Delete = 0)
BEGIN
  SET @SaveError = 778000003
  GOTO Cleanup
END

COMMIT TRAN

Cleanup:
 -- cleanup  
IF (@DocHandle IS NOT NULL)
   EXEC sp_xml_removedocument @DocHandle
   
IF (@SaveError != 0)
    BEGIN
    IF (XACT_STATE() != 0) BEGIN ROLLBACK TRAN END
    RAISERROR(@SaveError, 16, 1)
    RETURN 1
    END
RETURN 0

end
go

if object_id(N'Staging.AddManifest') is not null
  drop procedure Staging.AddManifest
go

create procedure Staging.AddManifest
  @ManagementPackId uniqueidentifier
 ,@MPName           nvarchar(256)
 ,@MPKeyToken       varchar(32)
 ,@DatasourceId     uniqueidentifier
 ,@Xml              xml
 ,@Delete           bit                   = 0
as
begin
DECLARE @IsSealed bit
DECLARE @DocHandle int
DECLARE @SaveError int
DECLARE @RowCount int
DECLARE @MPVersion nvarchar(20)
DECLARE @XmlText nvarchar(max)

SET @XmlText = CONVERT(nvarchar(max), @Xml)
EXEC sp_xml_preparedocument  @DocHandle OUTPUT, @XmlText ,
    N'<ROOT xmlns:pb="http://schemas.microsoft.com/mom/reporting/2007/ReportParameterSettings"/>'

BEGIN TRAN

IF LEN(@MPKeyToken) > 0 SET @IsSealed = 1 ELSE SET @IsSealed = 0

IF NOT EXISTS (SELECT * FROM Staging.Datasource WHERE DatasourceId = @DatasourceId)
BEGIN
  INSERT INTO Staging.Datasource (DatasourceId)
  SELECT @DatasourceId
  SET @SaveError = @@Error
  IF @SaveError <> 0 GOTO Cleanup
END

DELETE Staging.ManagementPackReferences WHERE ManagementPackIdReffedBy = @ManagementPackId
SET @SaveError = @@Error
IF @SaveError <> 0 GOTO Cleanup

IF @Delete = 1
BEGIN
  DELETE Staging.ColumnMapping WHERE ManagementPackId = @ManagementPackId --AND SourceColumnName = N'DisplayName'
  SET @SaveError = @@Error
  IF @SaveError <> 0 GOTO Cleanup

  DELETE Staging.ManagementPack WHERE ManagementPackId = @ManagementPackId
  SET @SaveError = @@Error
  IF @SaveError <> 0 GOTO Cleanup

  COMMIT TRAN
  GOTO Cleanup
END

SELECT @MPVersion = Version
    FROM OPENXML(@DocHandle, N'/Manifest/Identity', 2) 
WITH (  Version nvarchar(20))
SET @SaveError = @@Error
IF @SaveError <> 0 GOTO Cleanup

IF EXISTS (SELECT * FROM Staging.ManagementPack WHERE ManagementPackId = @ManagementPackId)
BEGIN
  UPDATE Staging.ManagementPack SET
    MPIsSealed = @IsSealed
   ,MPVersion = @MPVersion
   ,Manifest = @Xml
   ,LastModified = getutcdate()
  WHERE ManagementPackId = @ManagementPackId
  SET @SaveError = @@Error
  IF @SaveError <> 0 GOTO Cleanup
END
ELSE
BEGIN
  INSERT INTO Staging.[ManagementPack] (ManagementPackId, MPIsSealed, MPName, MPVersion, MPKeyToken, Manifest)
      SELECT @ManagementPackId, @IsSealed, @MPName, @MPVersion, @MPKeyToken, @Xml
  SET @SaveError = @@Error
  IF @SaveError <> 0 GOTO Cleanup
END

-- Load ManagementPackReferences - we need these to figure out TypeId's of any types used from other MP's
INSERT INTO Staging.[ManagementPackReferences] (ManagementPackIdSource,     ManagementPackIdReffedBy, AliasName)
    SELECT Staging.fn_MPId (ID, PublicKeyToken), @ManagementPackId, Alias
    FROM OPENXML(@DocHandle, N'/Manifest/References/Reference', 3)
    WITH (Alias nvarchar(256), ID nvarchar(256), PublicKeyToken nvarchar(32))
SET @SaveError = @@Error
IF @SaveError <> 0 GOTO Cleanup

SELECT @RowCount = count(*) FROM Staging.ManagementPack where ManagementPackId = @ManagementPackId
IF (@RowCount = 0 and @Delete = 0)
BEGIN
  SET @SaveError = 778000003
  GOTO Cleanup
END

COMMIT TRAN

Cleanup:
 -- cleanup  
IF (@DocHandle IS NOT NULL)
   EXEC sp_xml_removedocument @DocHandle
   
IF (@SaveError != 0)
    BEGIN
    IF (XACT_STATE() != 0) BEGIN ROLLBACK TRAN END
    RAISERROR(@SaveError, 16, 1)
    RETURN 1
    END
RETURN 0

end
go

if object_id(N'Staging.AddEnumerations') is not null
  drop procedure Staging.AddEnumerations
go

create procedure Staging.AddEnumerations
  @ManagementPackId uniqueidentifier
 ,@MPName           nvarchar(256)
 ,@MPKeyToken       varchar(32)
 ,@DatasourceId     uniqueidentifier
 ,@Xml              xml
 ,@Delete           bit                   = 0
as
begin
set nocount on
DECLARE @DocHandle int
DECLARE @SaveError int

EXEC sp_xml_preparedocument  @DocHandle OUTPUT, @Xml ,
    N'<ROOT xmlns:pb="http://schemas.microsoft.com/mom/reporting/2007/ReportParameterSettings"/>'

BEGIN TRAN

IF @Delete = 1
BEGIN
  DELETE Staging.EnumType WHERE ManagementPackId = @ManagementPackId
  SET @SaveError = @@Error
  IF @SaveError <> 0 GOTO Cleanup

  COMMIT TRAN
  GOTO Cleanup
END

INSERT INTO [Staging].[EnumType] (EnumTypeId, ManagementPackId, EnumTypeName, EnumTypeAccessibility, ParentEnumTypeId, Enabled, TimeAdded, LastModified)
	SELECT dbo.fn_MPObjectId(@MPName, @MPKeyToken, EnumTypeName), @ManagementPackId, EnumTypeName,
		Staging.fn_MapAccessibilityEnum(Accessibility),  dbo.fn_MPReferencedObjectId(@MPName, @MPKeyToken, ParentEnumTypeName), COALESCE(Enabled, 1), getutcdate(), getutcdate()
	FROM OPENXML (@DocHandle, '/EnumerationTypes/EnumerationValue', 3)
	WITH (EnumTypeName nvarchar(256) '@ID',
		   Accessibility		nvarchar(100),
		   Enabled		bit,
		   ParentEnumTypeName nvarchar(512) '@Parent')
   WHERE EnumTypeName not in (select EnumTypeName from Staging.EnumType where ManagementPackId = @ManagementPackId)

   SET @SaveError = @@Error
   IF @SaveError <> 0 GOTO Cleanup

COMMIT TRAN

Cleanup:
 -- cleanup  
IF (@DocHandle IS NOT NULL)
   EXEC sp_xml_removedocument @DocHandle
   
IF XACT_STATE() != 0 and @SaveError != 0
    BEGIN
    ROLLBACK TRAN
    RETURN 1
    END
RETURN 0

end
go

if object_id(N'Staging.GenerateExtractionConfigByMP') is not null
  drop procedure Staging.GenerateExtractionConfigByMP
go

create procedure [Staging].[GenerateExtractionConfigByMP]
 @DatasourceId     uniqueidentifier
,@ManagementPackId uniqueidentifier
,@TypeId           uniqueidentifier  = NULL
,@Disassociate     bit               = 0
,@DatasourceName   nvarchar(128)     = NULL
,@ProcessName      nvarchar(128)
as
begin
set nocount on
declare @SourceTable  nvarchar(128)
declare @SourceSchema nvarchar(128)
declare @TargetTable  nvarchar(128)
declare @TargetSchema nvarchar(128)
declare @Query        nvarchar(max)
declare @WMQuery      nvarchar(1024)
declare @error        int
declare @mpid         uniqueidentifier
declare @JobId        nvarchar(255)
declare @types        table (typeid nvarchar(255), viewname nvarchar(128))
declare @rels         table (typeid nvarchar(255), viewname nvarchar(128))

set @mpid = @ManagementPackId
select @JobId = lower(replace(convert(nvarchar(64), @DatasourceId), '-', '_'))
if @DatasourceName is not null set @JobId = replace(@DatasourceName, '-', '_')

-- if the MP has not be associated with this datasource, exit
if not exists (select * from Staging.ExtractionMP where ManagementPackId = @ManagementPackId and DatasourceId = @DatasourceId and IsDeleted = 0)
  and @Disassociate = 0
begin
  set @error = 778000001
  goto error
end

-- get a list of types included in the specified MP
insert @types
select a.ManagedTypeId, a.ManagedTypeViewName
from   Staging.ManagedType a
join   Staging.ExtractionType b on a.ManagedTypeId=b.TypeId and b.DatasourceId=@DatasourceId
where  a.ManagementPackId = @mpid
and    (b.IsDeleted = @Disassociate or b.IsActive = @Disassociate * -1)

if @TypeId is not null delete @types where typeid != @TypeId

-- get a list of relationships included in the specified MP
insert @rels 
select a.RelationshipTypeId, a.RelationshipTypeViewName
from   Staging.RelationshipType a
join   Staging.ExtractionType b on a.RelationshipTypeId=b.TypeId and b.DatasourceId=@DatasourceId
where  a.ManagementPackId = @mpid
and    (b.IsDeleted = @Disassociate or b.IsActive = @Disassociate * -1)

if @TypeId is not null delete @rels where typeid != @TypeId

-- find the source views that provide data for the types.
declare @extract table (SourceSchemaName nvarchar(128), SourceViewName nvarchar(128), TargetSchemaName nvarchar(128), TargetTableName nvarchar(128), ManagementPackId uniqueidentifier)
insert @extract
select distinct
 SourceSchemaName = c.SourceSchemaName
,SourceViewName   = c.SourceTableName
,TargetSchemaName = c.TargetSchemaName
,TargetTableName  = c.TargetTableName
,ManagementPackId = a.ManagementPackId
from  Staging.ManagedType a
join  Staging.ExtractionType b on a.ManagedTypeId=b.TypeId and b.DatasourceId=@DatasourceId
join  Staging.TableMapping c on a.ManagedTypeViewName=c.SourceTableName
where a.ManagedTypeViewName in (select viewname from @types)
and   a.TimeAdded = (select min(TimeAdded) from Staging.ManagedType where ManagedTypeViewName = a.ManagedTypeViewName)
and   b.IsDeleted = 0
and   b.IsActive  = 1

-- find the source views that provide data for the relationships
declare @stage table (SourceSchemaName nvarchar(128), SourceViewName nvarchar(128), TargetSchemaName nvarchar(128), TargetTableName nvarchar(128), ManagementPackId uniqueidentifier)
insert @stage
select distinct
 SourceSchemaName = c.SourceSchemaName
,SourceViewName   = c.SourceTableName
,TargetSchemaName = c.TargetSchemaName
,TargetTableName  = c.TargetTableName
,ManagementPackId = a.ManagementPackId
from  Staging.RelationshipType a
join  Staging.ExtractionType b on a.RelationshipTypeId=b.TypeId and b.DatasourceId=@DatasourceId
join  Staging.TableMapping c on a.RelationshipTypeViewName=c.SourceTableName
where a.RelationshipTypeViewName in (select viewname from @rels)
and   a.TimeAdded = (select min(TimeAdded) from Staging.RelationshipType where RelationshipTypeViewName = a.RelationshipTypeViewName)
and   b.IsDeleted = 0
and   b.IsActive  = 1

-- construct the config xml for the types.
declare @x1 xml, @x2 xml
set @x1 = (
select
 DatasourceId    = @DatasourceId
,SourceSchemaName= ClassExtraction.SourceSchemaName
,SourceViewName  = ClassExtraction.SourceViewName
,TargetSchemaName= ClassExtraction.TargetSchemaName
,TargetTableName = (select top 1 TargetTableName from @extract where SourceViewName = ClassExtraction.SourceViewName)
,[Columns]       = (select SourceTableName = [Column].SourceTableName, [Column].SourceColumnName, [Column].TargetTableName, [Column].TargetColumnName
                    from  Staging.ColumnMapping [Column]
                    join  Staging.ExtractionType et on [Column].TypeId=et.TypeId and et.DatasourceId=@DatasourceId
                    left join  Staging.ExtractionProperty ep on et.TypeId=ep.TypeId and et.DatasourceId=ep.DatasourceId and ep.PropertyId=[Column].PropertyId
                    where [Column].TargetTableName = ClassExtraction.TargetTableName
                    and   coalesce(ep.IsDeleted, 0) = 0 and coalesce(ep.IsActive, 1) = 1
                    and   et.IsDeleted = 0 and et.IsActive = 1
                    and   [Column].IsDropped = 0 and [Column].IsActive = 1
                    for xml auto, type)
,[Query]         = N'#' + ClassExtraction.SourceViewName + N'#QueryData#'
--,[WatermarkQuery]= N'#' + ClassExtraction.SourceViewName + N'#WMQueryData#'
from @extract ClassExtraction
for xml auto, elements)

-- construct the config xml for the relationships
set @x2 = (
select
 DatasourceId        = @DatasourceId
,SourceSchemaName    = RelationshipExtraction.SourceSchemaName
,SourceViewName      = RelationshipExtraction.SourceViewName
,TargetSchemaName    = RelationshipExtraction.TargetSchemaName
,TargetTableName     = (select top 1 TargetTableName from @stage where SourceViewName = RelationshipExtraction.SourceViewName)
,[Columns]           = (select SourceTableName = [Column].SourceTableName, [Column].SourceColumnName, [Column].TargetTableName, [Column].TargetColumnName
                        from  Staging.ColumnMapping [Column]
                        join  Staging.ExtractionType et on [Column].TypeId=et.TypeId and et.DatasourceId=@DatasourceId
                        left join  Staging.ExtractionProperty ep on et.TypeId=ep.TypeId and et.DatasourceId=ep.DatasourceId and ep.PropertyId=[Column].PropertyId
                        where [Column].TargetTableName = RelationshipExtraction.TargetTableName
                        and   coalesce(ep.IsDeleted, 0) = 0 and coalesce(ep.IsActive, 1) = 1
                        and   et.IsDeleted = 0 and et.IsActive = 1
                        and   [Column].IsDropped = 0 and [Column].IsActive = 1
                        for xml auto, type)
,[Query]             = N'#' + RelationshipExtraction.SourceViewName + N'#QueryData#'
--,[WatermarkQuery]    = N'#' + RelationshipExtraction.SourceViewName + N'#WMQueryData#'
from @stage RelationshipExtraction
for xml auto, elements)

-- put the type config into a vertex
declare @cls xml, @rel xml
set @cls = (
select ID = N'Extract_' + Staging.fn_GetFirstConcrete(p.value(N'SourceViewName[1]', N'nvarchar(128)')) + N'_' + @JobId
      ,ModuleName = N'Extract_' + Staging.fn_GetFirstConcrete(p.value(N'SourceViewName[1]', N'nvarchar(128)')) + N'_' + @JobId
      ,ModuleType = N'T-SQL'
      ,ModuleDescription = N'Extraction Job Step'
      ,ManagementPackId = (select ManagementPackId from @extract where SourceViewName = p.value(N'SourceViewName[1]', N'nvarchar(128)'))
      ,xmlns = N'http://schemas.microsoft.com/SystemCenter/ArtifactManagement/2007/09'
      ,convert(xml, N'<ClassExtraction xmlns="">' + convert(nvarchar(max), (select p.query(N'*'))) + N'</ClassExtraction>')
from @x1.nodes(N'*') Vertex(p)
for xml auto, type)

-- put the relationship config into a vertex
set @rel = (
select ID = N'Extract_' + Staging.fn_GetFirstConcrete(p.value(N'SourceViewName[1]', N'nvarchar(128)')) + N'_' + @JobId
      ,ModuleName = N'Extract_' + Staging.fn_GetFirstConcrete(p.value(N'SourceViewName[1]', N'nvarchar(128)')) + N'_' + @JobId
      ,ModuleType = N'T-SQL'
      ,ModuleDescription = N'Extraction Job Step'
      ,ManagementPackId = (select ManagementPackId from @extract where SourceViewName = p.value(N'SourceViewName[1]', N'nvarchar(128)'))
      ,xmlns = N'http://schemas.microsoft.com/SystemCenter/ArtifactManagement/2007/09'
      ,convert(xml, N'<RelationshipExtraction xmlns="">' + convert(nvarchar(max), (select p.query(N'*'))) + N'</RelationshipExtraction>')
from @x2.nodes(N'*') Vertex(p)
for xml auto, type)

-- construct the job document
declare @data  nvarchar(max)
set @data = N''
if @TypeId is null 
  begin
    set @data = N'<Job xmlns="http://schemas.microsoft.com/SystemCenter/ArtifactManagement/2007/09" '
    set @data = @data + N'ID="' + @ProcessName + N'">'
    set @data = @data + N'<Category>Extract</Category>'
    set @data = @data + N'<Description>Extraction Job: ' + @JobId + N'</Description>'
    set @data = @data + N'<IsRecurring>true</IsRecurring>'
    set @data = @data + N'<Vertices>'

    if @Disassociate = 0
      begin
        declare @src nvarchar(128), @tar nvarchar(128), @name nvarchar(255)
        declare @sschema nvarchar(128), @tschema nvarchar(128)
        declare @exTbl table (SourceSchema nvarchar(128), SourceTable  nvarchar(128), TargetSchema nvarchar(128), TargetTable nvarchar(128))
        insert @exTbl select RawSourceSchemaName, RawSourceTableName, RawTargetSchemaName, RawTargetTableName from Staging.RawTableExtract
        where MPName = (select MPName from Staging.ManagementPack where ManagementPackId = @ManagementPackId)
        and   coalesce(DatasourceId, @DatasourceId) = @DatasourceId

        -- add any raw tables that are configured for this datasource/mp
        while exists (select 1 from @exTbl)
          begin
            select top (1) @sschema = SourceSchema, @src = SourceTable, @tschema = TargetSchema, @tar = TargetTable from @exTbl
            set @name = N'Extract_' + @src + N'_' + @JobId

            set @data = @data + N'<Vertex '
            set @data = @data + N'xmlns="http://schemas.microsoft.com/SystemCenter/ArtifactManagement/2007/09" '
            set @data = @data + N'ID="' + @name + N'" '
            set @data = @data + N'ModuleName="' + @name + N'" '
            set @data = @data + N'ModuleType="T-SQL" '
            set @data = @data + N'ModuleDescription="Extraction Job Step" '
            set @data = @data + N'ManagementPackId="' + convert(nvarchar(64), @ManagementPackId) + N'">'
            set @data = @data + N'<TableExtraction xmlns="">'
            set @data = @data + N'<DatasourceId>' + convert(nvarchar(64), @DatasourceId) + N'</DatasourceId>'
            set @data = @data + N'<SourceSchemaName>' + @sschema + N'</SourceSchemaName>'
            set @data = @data + N'<SourceViewName>' + @src + N'</SourceViewName>'
            set @data = @data + N'<TargetSchemaName>' + @tschema + N'</TargetSchemaName>'
            set @data = @data + N'<TargetTableName>' + @tar + N'</TargetTableName>'
            set @data = @data + N'<Columns>'

            -- schema definition for raw tables
            select @data = @data + N'<Column SourceTableName="' + TABLE_NAME + N'" SourceColumnName="' + COLUMN_NAME + N'" TargetTableName="' + TABLE_NAME + '" TargetColumnName="' + COLUMN_NAME + N'" />'
            from   INFORMATION_SCHEMA.COLUMNS 
            where  TABLE_NAME = @tar 
            and    TABLE_SCHEMA = @tschema
	    and    COLUMN_NAME NOT IN ( N'DWTimeStamp', N'BatchId', N'InsertedTimestamp', N'RejectedCount')
            order by ORDINAL_POSITION

            set @data = @data + N'</Columns>'
            set @data = @data + N'<Query>#' + @src + N'#QueryData#</Query>'
            set @data = @data + N'</TableExtraction>'
            set @data = @data + N'</Vertex>'

            insert @extract (SourceSchemaName, SourceViewName, TargetSchemaName, TargetTableName) 
            values (@sschema, @src, @tschema, @tar)
            delete @exTbl where SourceTable = @src and TargetTable = @tar
          end
      end
  end

set @data = @data + coalesce(convert(nvarchar(max), @cls), '')
set @data = @data + coalesce(convert(nvarchar(max), @rel), '')

if @TypeId is null
  begin
    set @data = @data + N'</Vertices><Links /></Job>'
  end

-- get the extract query for each type.
declare @result table (Result nvarchar(max))
while exists (select 1 from @extract)
  begin
    select top (1) @SourceTable = SourceViewName, 
                   @SourceSchema = SourceSchemaName, 
                   @TargetTable = TargetTableName, 
                   @TargetSchema = TargetSchemaName 
    from @extract
    insert @result exec Staging.GenerateQuery @SourceTable, 
                                              @SourceSchema,
                                              @TargetTable,
                                              @TargetSchema,
                                              @DatasourceId, 
                                              1, 
                                              @WMQuery out
    select @Query = Result from @result
    if @Query is not null 
      begin
        set @Query = N'<![CDATA[' + @Query + N']]>'
        set @WMQuery = N'<![CDATA[' + @WMQuery + N']]>'
        set @data = replace(@data, N'#' + @SourceTable + N'#QueryData#', @Query)
        set @data = replace(@data, N'#' + @SourceTable + N'#WMQueryData#', @WMQuery)
      end

    select @Query = null, @WMQuery = null
    delete @result
    delete @extract where SourceViewName = @SourceTable
  end

-- get the extract query for each relationship
while exists (select 1 from @stage)
  begin
    select top (1) @SourceTable = SourceViewName from @stage
    insert @result exec Staging.GenerateQuery @SourceTable, 
                                              NULL,
                                              NULL,
                                              NULL,
                                              @DatasourceId, 
                                              1, 
                                              @WMQuery out
    select @Query = Result from @result
    if @Query is not null 
      begin
        set @Query = N'<![CDATA[' + @Query + N']]>'
        set @WMQuery = N'<![CDATA[' + @WMQuery + N']]>'
        set @data = replace(@data, N'#' + @SourceTable + N'#QueryData#', @Query)
        set @data = replace(@data, N'#' + @SourceTable + N'#WMQueryData#', @WMQuery)
      end

    delete @result
    delete @stage where SourceViewName = @SourceTable
  end

--select JobXml = case when len(@data) > 0 then convert(xml, @data) else NULL end
select JobXml = case when len(@data) > 0 then @data else NULL end

error:
  if @error != 0 raiserror(@error, 16, 1)

return
end
go

/*
* Special casing this due to bug# 207111

declare @wm nvarchar(max) = '', @ds uniqueidentifier = '07CA4239-16FA-8C38-0F7E-30EB21B42CFA'
exec Staging.GenerateQuery 'MTV_System$DeviceHasSoftwareUpdateInstalled', null, null, null, @ds, 1, @wm output
print @wm
exec Staging.GenerateQuery 'MTV_System$WorkItemAboutConfigItem', null, null, null, @ds, 1, @wm output
print @wm
*/

if object_id(N'Staging.GenerateQuery') is not null
  drop procedure Staging.GenerateQuery
go

create procedure Staging.GenerateQuery
  @SourceTable   nvarchar(128)
 ,@SourceSchema  nvarchar(128)
 ,@TargetTable   nvarchar(128) 
 ,@TargetSchema  nvarchar(128)
 ,@DatasourceId  uniqueidentifier
 ,@SelectOnly    bit               = 1
 ,@WMQuery       nvarchar(1024)    = null  out
 --,@ParameterList nvarchar(max)     = N'DatasourceId'
as
begin
set nocount on

declare @idx       int
declare @separ     nvarchar(10)
declare @tgt       nvarchar(128)
declare @pk        nvarchar(128)
declare @tables    nvarchar(max)
declare @insert    nvarchar(max)
declare @cols      nvarchar(max)
declare @where     nvarchar(max)
declare @id        nvarchar(128)
declare @schema    nvarchar(128)
declare @Filter    nvarchar(128)
declare @orderBy   nvarchar(128)
declare @Watermark nvarchar(128)
declare @query     nvarchar(max)
declare @list      table (sch nvarchar(128), target nvarchar(128), SourceQuery nvarchar(max) null)
declare @types     table (sch nvarchar(128), target nvarchar(128))
declare @source    table (SourceTable nvarchar(128), SourceColumn nvarchar(255))
set @Watermark = N'LastModified'


-- one off handling of raw tables.
if (select COUNT(*) from Staging.RawTableExtract where RawSourceTableName = @SourceTable) > 0
begin
  -- staging query for raw tables
  set @query = N'SELECT TOP (@BatchSize) WITH TIES '
  select @query = @query + N'[' + COLUMN_NAME + N'],'
  from   INFORMATION_SCHEMA.COLUMNS 
  where  TABLE_NAME = @TargetTable 
  and    TABLE_SCHEMA = @TargetSchema
  and    COLUMN_NAME NOT IN ( N'DWTimeStamp', N'BatchId', N'InsertedTimestamp', N'RejectedCount')
  and    COLUMN_NAME != N'DatasourceId'
  order by ORDINAL_POSITION
  
  set @query = @query + N'@DatasourceId'
  set @query = @query + N' FROM [' + @SourceSchema + N'].[' + @SourceTable + N']'
  set @query = @query + N' WHERE [' + @SourceTable + N'].[' + @Watermark  + N'] > @MinWaterMark'
  set @query = @query + N' AND [' + @SourceTable + N'].[' + @Watermark + N'] <= @MaxWaterMark'
  set @query = @query + N' ORDER BY [' + @SourceTable + N'].[' + @Watermark + N']'
  
  -- the watermark reset query only for completeness.
  set @WMQuery = N'select max(' + @Watermark + N') from ' + @TargetSchema + N'.[' + @TargetTable + N'] where DatasourceId = ''' + convert(nvarchar(64), @DatasourceId) + N''''
  
  select @query
  -- since it is a raw table, we are done.
  return
end

insert @types
select TargetSchemaName, TargetTableName
from   Staging.ColumnMapping a
left join Staging.ManagedType b on a.TypeId=b.ManagedTypeId
left join Staging.RelationshipType c on a.TypeId=c.RelationshipTypeId
where a.SourceTableName like @SourceTable
group by TargetTableName, TargetSchemaName

select @cols = N'', @tables = N'', @insert = N'', @where = N'', @separ = N'', @idx = 0
while exists (select 1 from @types)
  begin
    select top 1 @schema = sch, @id = target from @types

    insert @list select SourceSchemaName, SourceTableName, null
                 from Staging.ColumnMapping where TargetTableName = @id
                 and  SourceTableName is not null
                 group by SourceSchemaName, SourceTableName

    update @list set SourceQuery = Staging.fn_GetQueryTextForRelationshipWithProperites(null, sch, target)

    select @tables = @tables + N',' +
        case
            when SourceQuery is null then sch + N'.[' + target + N']'
            else replace(replace('(%QUERY%) as [%SOURCETABLENAME%]', '%QUERY%', SourceQuery), '%SOURCETABLENAME%', target)
        end
    from @list

    insert @source select N'[' + a.SourceTableName + N']', N'[' + a.SourceColumnName + N']'
    from Staging.ColumnMapping a
    left join Staging.ExtractionType b on a.TypeId=b.TypeId and b.DatasourceId=@DatasourceId
    left join Staging.ExtractionProperty c on a.TypeId=c.TypeId and a.PropertyId=c.PropertyId and c.DatasourceId=@DatasourceId
    where a.TargetTableName = @id and a.IsDropped = 0 and a.IsActive = 1
    and   b.IsDeleted = 0 and b.IsActive = 1
    and   c.IsDeleted = 0 and c.IsActive = 1
    order by a.SourceColumnName

    --if (select count(*) from @source) > 0
    --begin
      insert @source select N'[' + a.SourceTableName + N']', case when (a.SourceColumnName = N'DatasourceId' OR (a.SourceColumnName = N'BatchId' and a.SourceTableName IS NULL)) then N'@' + a.SourceColumnName else N'[' + a.SourceColumnName + N']' end
      from Staging.ColumnMapping a where a.TargetTableName = @id 
      and (a.SourceTableName in (N'BaseManagedEntity', N'Relationship') or 
           a.SourceTableName is null or 
           a.SourceColumnName IN( N'BaseManagedEntityId' , N'RelationshipId' ,N'SourceTypeId', N'TargetTypeId'))
      order by a.SourceColumnName
    --end

    select @insert = @insert + N',[' + a.TargetColumnName + N']'
    from Staging.ColumnMapping a
    left join Staging.ExtractionType b on a.TypeId=b.TypeId and b.DatasourceId=@DatasourceId
    left join Staging.ExtractionProperty c on a.TypeId=c.TypeId and a.PropertyId=c.PropertyId and c.DatasourceId=@DatasourceId
    where a.TargetTableName = @id and a.IsDropped = 0 and a.IsActive = 1 
    and   b.IsDeleted = 0 and b.IsActive = 1
    and   c.IsDeleted = 0 and c.IsActive = 1
    order by a.SourceColumnName

    --if len(@insert) > 0
    --begin
      select @insert = @insert + N',[' + a.TargetColumnName + N']'
      from Staging.ColumnMapping a where a.TargetTableName = @id 
      and (a.SourceTableName in (N'BaseManagedEntity', N'Relationship') or 
           a.SourceTableName is null or 
           a.SourceColumnName IN( N'BaseManagedEntityId' , N'RelationshipId' ,N'SourceTypeId', N'TargetTypeId'))
      order by a.SourceColumnName
    --end

    select @cols = @cols + ',' + coalesce(SourceTable + N'.' + SourceColumn, SourceColumn) from @source

    set @insert = ' ' + substring(@insert, 2, len(@insert))
    set @cols = ' ' + substring(@cols, 2, len(@cols))
    set @tables = substring(@tables, 2, len(@tables))

    if charindex(N'BaseManagedEntity', @tables) > 0 
      begin
        set @pk = N'BaseManagedEntityId'
        set @Filter = N' and BaseManagedEntity.' + @Watermark + N' > @MinWaterMark'
        set @Filter = @Filter + N' and BaseManagedEntity.' + @Watermark + N' <= @MaxWaterMark'
        set @orderBy = N'BaseManagedEntity.[' + @Watermark + N']'
      end
    if charindex(N'Relationship', @tables) > 0 
      begin
        set @pk = N'RelationshipId'
        set @Filter = N' and Relationship.' + @Watermark + N' > @MinWaterMark'
        set @Filter = @Filter + N' and Relationship.' + @Watermark + N' <= @MaxWaterMark'
        set @orderBy = N'Relationship.[' + @Watermark + N']'
      end
    set @WMQuery = N'select max(' + @Watermark + N') from ' + @schema + N'.[' + @id + N'] where DatasourceId = ''' + convert(nvarchar(64), @DatasourceId) + N''''

    while exists (select 1 from @list)
      begin
        select top 1 @tgt = target, @idx = @idx + 1 from @list

        set @where = @where + @separ + N'[' + @tgt + N'].[' + @pk + N']'
        if @idx % 2 = 1 set @separ = N'=' else set @separ = N' and '

        delete @list where target = @tgt
      end
    if @pk = N'BaseManagedEntityId'
      begin
        declare @x nvarchar(max), @clist nvarchar(max), @wlist nvarchar(max)
        select @clist = N'', @wlist = N''
        select @clist = @clist + N',' + SourceColumn from @source where SourceTable is null and SourceColumn != N'@DatasourceId'
        set @clist = substring(@clist, 2, len(@clist))

        select @wlist = @wlist + N' or ' + a.SourceColumn + N'=1'
        from @source a join Staging.ExtractionType b on a.SourceColumn=N'['+b.TypeName+N']' 
        where a.SourceTable is null and a.SourceColumn != N'@DatasourceId' 
        and b.IsActive=1 and b.IsDeleted=0 and b.DatasourceId=@DatasourceId
        set @wlist = substring(@wlist, 4, len(@wlist))

        set @x = N'(select top (@BatchSize) with ties BaseManagedEntityId = BaseManagedEntity.BaseManagedEntityId, ManagedEntityId = TypedManagedEntity.BaseManagedEntityId, TypeName = ManagedType.TypeName '
        set @x = @x + N' from   dbo.ManagedType'
        set @x = @x + N' join   dbo.TypedManagedEntity on ManagedType.ManagedTypeId=TypedManagedEntity.ManagedTypeId'
        set @x = @x + N' join   dbo.BaseManagedEntity on TypedManagedEntity.BaseManagedEntityId=BaseManagedEntity.BaseManagedEntityId'
        set @x = @x + N' where  ManagedType.ManagedTypeViewName = ''' + @SourceTable + N''''
        set @x = @x + N' /*watermark*/'
        set @x = @x + N' order by ' + @orderBy + ') p'
        set @x = @x + N' pivot (count(p.ManagedEntityId) for p.TypeName in (' + @clist + N')) pvt,'
        set @tables = @x + @tables
        set @where = @where + @separ + replace(N'[BaseManagedEntity].$t$=pvt.$t$', N'$t$', @pk)
        set @where = @where + @separ + N'(' + @wlist + N')'
      end

    set @where = @where + N' /*watermark*/'
    set @query = N''
    --set @query = N'DECLARE @DatasourceId uniqueidentifier '
    --set @query = @query + N'DECLARE @Watermark datetime '
    --set @query = @query + N'set @DatasourceId = ''' + convert(nvarchar(60), @DatasourceId) + N''' '
    --set @query = @query + N'set @Watermark = ''17530101'' '
    if @SelectOnly = 0
    begin
      set @query = @query + N'INSERT INTO ' + db_name() + N'.' + @schema + '.[' + @id + N'] ( '
      set @query = @query + @insert + N') '
    end
    set @query = @query + N'SELECT '
    set @query = @query + @cols + ' '
    set @query = @query + N'FROM ' + @tables + ' '
    set @query = @query + N'WHERE ' + @where
    set @query = replace(@query, N'/*watermark*/', coalesce(@Filter, N''))
    --set @query = @query + N' ORDER BY ' + @orderBy
    if len(@insert) = 0 or len(@cols) = 0
      select null
    else
      select @query

    delete @types where target = @id
    select @cols = N'', @tables = N'', @insert = N'', @where = N'', @separ = N'', @idx = 0
  end
end
go

/*

declare @SourceTable  nvarchar(128)
       ,@DatasourceId uniqueidentifier

set @SourceTable = 'MTV_Computer'
--set @SourceTable = 'MTV_ComputerPrimaryUser'
--set @SourceTable = '%%'
set @DatasourceId = 'E0D20408-7A5D-4004-80DA-30D5AC898455'
exec Staging.GenerateQuery
 @SourceTable
,@DatasourceId
,1

declare @packs table (MPId uniqueidentifier)
declare @dsid uniqueidentifier
       ,@id   uniqueidentifier
set @dsid = 'E0D20408-7A5D-4004-80DA-30D5AC898455'
--set @dsid = '68FE3B7E-4BD4-43B7-A912-BC091ABB5F6F'
insert @packs select ManagementPackId from staging.managementpack order by timeadded
while exists (select 1 from @packs)
begin
  select top 1 @id = MPId from @packs

  exec Staging.AssociateMP
   @DatasourceId=@dsid
  ,@ManagementPackId=@id
  ,@Disassociate=0

  delete @packs where MPId = @id
end
*/

if object_id(N'Staging.GenerateColumnMap') is not null
  drop procedure Staging.GenerateColumnMap
go

create procedure Staging.GenerateColumnMap
  @SourceTable   nvarchar(128)
 ,@DatasourceId  uniqueidentifier
as
begin
set nocount on
declare @TargetTable  nvarchar(128)

select top (1) @TargetTable = TargetTableName from Staging.ColumnMapping where SourceTableName = @SourceTable

select a.SourceTableName, a.SourceColumnName, a.TargetTableName, a.TargetColumnName
from   Staging.ColumnMapping a
left join Staging.ExtractionType b on a.TypeId=b.TypeId 
  and b.DatasourceId=@DatasourceId 
  and b.IsActive=1 
  and b.IsDeleted=0
left join Staging.ExtractionProperty c on a.PropertyId=c.PropertyId 
  and b.TypeId=c.TypeId 
  and c.DatasourceId=@DatasourceId 
  and c.IsActive=1 
  and c.IsDeleted=0
where  a.TargetTableName = @TargetTable
  and a.IsDropped = 0 
  and a.IsActive = 1
end
go

-- Dw Staging Area and Dw Cmdb may be separate.
-- The list of tables included in the grooming operation may not be exhaustive.
if OBJECT_ID(N'Staging.GroomDwStagingData') is not null
  drop procedure Staging.GroomDwStagingData
go

CREATE PROCEDURE Staging.GroomDwStagingData 
(
    @TargetId uniqueidentifier,
    @RetentionPeriodInMinutes int,
    @GroomingCriteria nvarchar(max),
    @BatchSize int
)
AS
BEGIN
SET NOCOUNT ON

DECLARE @Config       xml
DECLARE @View         nvarchar(128)
DECLARE @Query        nvarchar(max)
DECLARE @start        datetime
DECLARE @Iter         int
DECLARE @rows         int
DECLARE @error        int
DECLARE @items table (ModuleName nvarchar(255),
                      WarehouseEntityName nvarchar(255),
                      RequiredWarehouseEntityName nvarchar(255),
                      ManagedTypeViewName nvarchar(255),
                      Watermark datetime)

IF LEN(@GroomingCriteria) <= 0
  RETURN

SET @Config = CONVERT(xml, @GroomingCriteria)
IF @BatchSize = 0
  SET @BatchSize = null  
-- with shredded config
INSERT @items
SELECT
 ModuleName = p.value(N'child::ModuleName[1]', N'nvarchar(255)')
,WarehouseEntityName = p.value(N'child::WarehouseEntityName[1]', N'nvarchar(255)')
,RequiredWarehouseEntityName =p.value(N'child::RequiredWarehouseEntityName[1]', N'nvarchar(255)')
,ManagedTypeViewName = p.value(N'child::ManagedTypeViewName[1]', N'nvarchar(255)')
,Watermark = p.value(N'child::Watermark[1]', N'datetime')
FROM   @Config.nodes(N'/Config/*') Elem(p)

-- add tables that are not configured for any transform, using a default timestamp.
IF (SELECT COUNT(*) FROM @items) > 0
BEGIN
  DECLARE @offset datetime
  SELECT @offset = MAX(Watermark) FROM @items
  IF @offset > CONVERT(datetime, 0)
    SET @offset = @offset - 2

  INSERT @items
  SELECT 'IGNORED', TABLE_NAME, TABLE_NAME, TABLE_NAME, @offset
  FROM   INFORMATION_SCHEMA.TABLES
  WHERE TABLE_SCHEMA = 'inbound'
  AND   TABLE_NAME NOT IN (SELECT ManagedTypeViewName FROM @items GROUP BY ManagedTypeViewName)
  AND   TABLE_TYPE = 'BASE TABLE'
END

/*
****************************************************************************************************
SP1 Fix for Bug 183978 - Outrigger transforms will not transform 3rd level enums when EnumType table is 
		      groomed out.  SP1 Fix is to disable grooming of this table since rowcount is comparatively
		      low for this table.  If grooming is re-enabled for inbound.enumtype, outrigger 
		      transform must be modified to pick up 3rd level enums after enums have been groomed out.
****************************************************************************************************
*/
DELETE @items
WHERE ManagedTypeViewName IN ('EnumType', 'RelationshipType')

SET DEADLOCK_PRIORITY LOW
-- Staging Db Query
WHILE EXISTS (SELECT 1 FROM @items)
BEGIN
  SELECT TOP (1) @View = ManagedTypeViewName, @Query = 
         N'IF OBJECT_ID(''inbound.' + ManagedTypeViewName + N''') IS NOT NULL ' +
         N'DELETE ' + coalesce(N'TOP (' +convert(nvarchar(20), @BatchSize) + N')', N'') + 
         N' inbound.' + ManagedTypeViewName + 
         N' WHERE DWTimestamp < ''' + convert(nvarchar(30), MIN(Watermark), 121) + 
         N''''
  FROM @items a
  GROUP BY ManagedTypeViewName
  
  SELECT @Iter = 0, @rows = -1, @start = GETUTCDATE()
  WHILE @Iter <= 3 AND @rows != 0
  BEGIN
    EXEC(@Query)
    SELECT @error = @@ERROR, @rows = @@ROWCOUNT, @Iter = DATEDIFF(ss, @start, GETUTCDATE())
    IF @error != 0
      RETURN 1
  END
  
  DELETE @items WHERE ManagedTypeViewName = @View
END

RETURN 0
END
go

