if object_id('Staging.Datasource') is null
create table Staging.Datasource (
    [DatasourceId]  [uniqueidentifier]  NOT NULL,
    [TimeAdded]     [datetime]          NOT NULL CONSTRAINT [DF_Staging_Datasource_TimeAdded]  DEFAULT (getutcdate()),
    [LastModified]  [datetime]          NOT NULL CONSTRAINT [DF_Staging_Datasource_LastModified]  DEFAULT (getutcdate()),
    CONSTRAINT [PK_Staging_Datasource] PRIMARY KEY CLUSTERED (DatasourceId),
)
go

if object_id('Staging.RawTableExtract') is null
create table Staging.RawTableExtract (
  RawSourceSchemaName       nvarchar(128)        not null ,
  RawSourceTableName        nvarchar(128)        not null ,
  RawTargetSchemaName       nvarchar(128)        not null ,
  RawTargetTableName        nvarchar(128)        not null ,
  MPName                    nvarchar(255)        not null ,
  DatasourceId              uniqueidentifier     null ,
  LastModified              datetime             not null ,
  CONSTRAINT PK_Staging_RawTableExtract PRIMARY KEY CLUSTERED (RawSourceTableName, RawTargetTableName) ,
  CONSTRAINT FK_Staging_RawTableExtract_Datasource FOREIGN KEY (DatasourceId) REFERENCES Staging.Datasource(DatasourceId) ,
)
go

if object_id('Staging.ManagementPack') is null
create table Staging.ManagementPack (
    [ManagementPackId] [uniqueidentifier] NOT NULL,
    [MPIsSealed] [bit] NOT NULL,
    [MPName] [nvarchar](256) NOT NULL,
    [MPVersion] [nvarchar](20) NULL,
    [MPKeyToken] [varchar](32) NULL,
    [Manifest] [xml] NULL,
    [TimeAdded] [datetime] NOT NULL CONSTRAINT [DF_Staging_ManagementPack_TimeAdded]  DEFAULT (getutcdate()),
    [LastModified] [datetime] NOT NULL CONSTRAINT [DF_Staging_ManagementPack_LastModified]  DEFAULT (getutcdate()),
    CONSTRAINT [PK_Staging_ManagementPack] PRIMARY KEY CLUSTERED ([ManagementPackId]) ,
)
go

if object_id('Staging.ManagementPackReferences') is null
create table Staging.ManagementPackReferences (
    [ManagementPackIdSource] [uniqueidentifier] NOT NULL,
    [ManagementPackIdReffedBy] [uniqueidentifier] NOT NULL,
    [AliasName] [nvarchar](256) NOT NULL,
    [TimeAdded] [datetime] NOT NULL CONSTRAINT [DF_Staging_ManagementPackReferences_TimeAdded] DEFAULT (getutcdate()),
    [LastModified]   [datetime]     NOT NULL CONSTRAINT [DF_Staging_ManagementPackReferences_LastModified] DEFAULT (getutcdate()),
    CONSTRAINT FK_Staging_ManagementPackReference_ManagementPackIdSource FOREIGN KEY (ManagementPackIdSource) REFERENCES Staging.ManagementPack (ManagementPackId) ,
    CONSTRAINT FK_Staging_ManagementPackReference_ManagementPackidReffedBy foreign key (ManagementPackIdReffedBy) REFERENCES Staging.ManagementPack (ManagementPackId) ,
)
go

if object_id('Staging.ColumnMapping') is null
create table Staging.ColumnMapping (
    ColumnMappingId       uniqueidentifier       not null CONSTRAINT DF_Staging_ColumnMapping_ColumnMappingId DEFAULT (newid())
   ,TypeId                uniqueidentifier       not null
   ,PropertyId            uniqueidentifier       not null
   ,SourceSchemaName      nvarchar(128)          null
   ,SourceTableName       nvarchar(128)          null
   ,SourceColumnName      nvarchar(128)          not null
   ,TargetSchemaName      nvarchar(128)          not null
   ,TargetTableName       nvarchar(128)          not null
   ,TargetColumnName      nvarchar(128)          not null
   ,IsDropped             bit                    not null
   ,IsActive              bit                    not null
   ,ManagementPackId      uniqueidentifier       not null
   ,TimeAdded             datetime               not null CONSTRAINT DF_Staging_ColumnMapping_TimeAdded DEFAULT (getutcdate())
   ,LastModified          datetime               not null CONSTRAINT DF_Staging_ColumnMapping_LastModified DEFAULT (getutcdate())
   ,CONSTRAINT PK_Staging_ColumnMapping PRIMARY KEY CLUSTERED (ColumnMappingId)
   ,CONSTRAINT FK_Staging_ColumnMapping_ManagementPack FOREIGN KEY (ManagementPackId) REFERENCES Staging.ManagementPack (ManagementPackId)
)
go

if object_id(N'Staging.TableMapping') is null
create table Staging.TableMapping (
    TypeId                uniqueidentifier       not null
   ,SourceSchemaName      nvarchar(128)          not null
   ,SourceTableName       nvarchar(128)          not null
   ,TargetSchemaName      nvarchar(128)          not null
   ,TargetTableName       nvarchar(128)          not null
   ,TimeAdded             datetime               not null CONSTRAINT DF_Staging_TableMapping_TimeAdded DEFAULT (getutcdate())
   ,LastModified          datetime               not null CONSTRAINT DF_Staging_TableMapping_LastModified DEFAULT (getutcdate())
   ,CONSTRAINT PK_Staging_TableMapping PRIMARY KEY CLUSTERED (TypeId)
   ,CONSTRAINT UQ_Staging_TableMapping_TargetTableName UNIQUE (TargetTableName)
)
go

if object_id('Staging.ManagedType') is null
create table Staging.ManagedType (
    [ManagedTypeId] [uniqueidentifier] NOT NULL,
    [TypeName] [nvarchar](256) NOT NULL,
    [BaseManagedTypeId] [uniqueidentifier] NULL,
    [ManagementPackId] [uniqueidentifier] NOT NULL,
    [IsHosted] [bit] NOT NULL,
    [IsAbstract] [bit] NOT NULL,
    [Accessibility] [tinyint] NOT NULL,
    [IsSingleton] [bit] NOT NULL,
    [IsSealed] [bit] NOT NULL,
    [ManagedTypeTableName] [nvarchar](128) NULL,
    [ManagedTypeViewName] [nvarchar](128) NULL,
    [ParentFolderId] [uniqueidentifier] NULL,
    [Comment] [nvarchar](255) NULL,
    [TimeAdded] [datetime] NOT NULL CONSTRAINT [DF_Staging_ManagedType_TimeAdded]  DEFAULT (getutcdate()),
    [LastModified] [datetime] NOT NULL CONSTRAINT [DF_Staging_ManagedType_LastModified]  DEFAULT (getutcdate()),
    [ModifiedBy] [nvarchar](255) NULL CONSTRAINT [DF_Staging_ManagedType_ModifiedBy]  DEFAULT (user_name()),
    CONSTRAINT [PK_Staging_ManagedType] PRIMARY KEY CLUSTERED (ManagedTypeId),
    CONSTRAINT FK_Staging_ManagedType_ManagementPackId FOREIGN KEY (ManagementPackId) REFERENCES Staging.ManagementPack (ManagementPackId),
    CONSTRAINT FK_Staging_ManagedType_BaseManagedTypeId FOREIGN KEY (BaseManagedTypeId) REFERENCES Staging.ManagedType (ManagedTypeId),
)
go

if object_id('Staging.ManagedTypeProperty') is null
create table Staging.ManagedTypeProperty (
    [ManagedTypePropertyId] [uniqueidentifier] NOT NULL,
    [ManagedTypeId] [uniqueidentifier] NOT NULL,
    [ManagedTypePropertyName] [nvarchar](256) NOT NULL,
    [IsPrimaryKey] [bit] NOT NULL CONSTRAINT [DF_Staging_ManagedTypeProperty_IsPrimaryKey]  DEFAULT ((0)),
    [IsCaseSensitive] [bit] NULL,
    [PropertyType] [tinyint] NOT NULL,
    [PropertyLength] [int] NULL,
    [PropertyMinLength] [int] NULL,
    [PropertyScale] [tinyint] NULL,
    [ColumnName] [nvarchar](128) NULL,
    [ColumnOrder] [int] NULL,
    [InternalOrder] [int] NULL,
    [ManagementPackId] [uniqueidentifier] NOT NULL,
    [Comment] [nvarchar](255) NULL,
    [TimeAdded] [datetime] NOT NULL CONSTRAINT [DF_Staging_ManagedTypeProperty_TimeAdded]  DEFAULT (getutcdate()),
    [LastModified] [datetime] NOT NULL CONSTRAINT [DF_Staging_ManagedTypeProperty_LastModified]  DEFAULT (getutcdate()),
    [ModifiedBy] [nvarchar](255) NULL CONSTRAINT [DF_Staging_ManagedTypeProperty_ModifiedBy]  DEFAULT (user_name()),
    [Required] [bit] NULL,
    [Optimized] [bit] NULL,
    [MinValue] [int] NULL,
    [MaxValue] [int] NULL,
    [RegEx] [nvarchar](255) NULL,
    [EnumTypeId] [uniqueidentifier] NULL,
    [DefaultValue] [nvarchar](2000) NULL,
    CONSTRAINT [PK_Staging_ManagedTypeProperty] PRIMARY KEY CLUSTERED ([ManagedTypePropertyId]) ,
    CONSTRAINT FK_Staging_ManagedTypeProperty_ManagedTypeId FOREIGN KEY (ManagedTypeId) REFERENCES Staging.ManagedType (ManagedTypeId) ,
    CONSTRAINT FK_Staging_ManagedTypeProperty_ManagementPackId FOREIGN KEY (ManagementPackId) REFERENCES Staging.ManagementPack (ManagementPackId) ,
)
go

if object_id('Staging.RelationshipType') is null
create table Staging.RelationshipType (
    [RelationshipTypeId] [uniqueidentifier] NOT NULL,
    [RelationshipTypeName] [nvarchar](256) NOT NULL,
    [BaseRelationshipTypeId] [uniqueidentifier] NULL,
    [SourceManagedTypeId] [uniqueidentifier] NOT NULL,
    [TargetManagedTypeId] [uniqueidentifier] NOT NULL,
    [ManagementPackId] [uniqueidentifier] NOT NULL,
    [IsAbstract] [bit] NOT NULL,
    [Accessibility] [tinyint] NOT NULL,
    [IsSealed] [bit] NOT NULL,
    [IsCycleOk] [bit] NOT NULL,
    [RelationshipTypeTableName] [nvarchar](128) NULL,
    [RelationshipTypeViewName] [nvarchar](128) NULL,
    [Comment] [nvarchar](255) NULL,
    [TimeAdded] [datetime] NOT NULL CONSTRAINT [DF_Staging_RelationshipType_TimeAdded]  DEFAULT (getutcdate()),
    [LastModified] [datetime] NOT NULL CONSTRAINT [DF_Staging_RelationshipType_LastModified]  DEFAULT (getutcdate()),
    [ModifiedBy] [nvarchar](255) NULL CONSTRAINT [DF_Staging_RelationshipType_ModifiedBy]  DEFAULT (user_name()),
    [SourceName] [nvarchar](256) NULL,
    [SourceId] [uniqueidentifier] NULL,
    [TargetName] [nvarchar](256) NULL,
    [TargetId] [uniqueidentifier] NULL,
    [SourceMaxCardinality] [int] NULL,
    [TargetMaxCardinality] [int] NULL,
    [SourceMinCardinality] [int] NULL,
    [TargetMinCardinality] [int] NULL,
    CONSTRAINT [PK_Staging_RelationshipType] PRIMARY KEY NONCLUSTERED ([RelationshipTypeId]),
    CONSTRAINT FK_Staging_RelationshipType_ManagementPackId FOREIGN KEY (ManagementPackId) REFERENCES Staging.ManagementPack (ManagementPackId),
    CONSTRAINT FK_Staging_RelationshipType_BaseRelationshipTypeId FOREIGN KEY (BaseRelationshipTypeId) REFERENCES Staging.RelationshipType (RelationshipTypeId),
    CONSTRAINT FK_Staging_RelationshipType_SourceManagedTypeId FOREIGN KEY (SourceManagedTypeId) REFERENCES Staging.ManagedType (ManagedTypeId) ,
    CONSTRAINT FK_Staging_RelationshipType_TargetManagedTypeId FOREIGN KEY (TargetManagedTypeId) REFERENCES Staging.ManagedType (ManagedTypeId) ,
)
go

if object_id('Staging.RelationshipTypeProperty') is null
create table Staging.RelationshipTypeProperty (
    [RelationshipPropertyId] [uniqueidentifier] NOT NULL,
    [RelationshipTypeId] [uniqueidentifier] NOT NULL,
    [RelationshipPropertyName] [nvarchar](256) NOT NULL,
    [IsCaseSensitive] [bit] NULL,
    [PropertyType] [tinyint] NOT NULL,
    [PropertyLength] [int] NULL,
    [PropertyScale] [tinyint] NULL,
    [ColumnName] [nvarchar](128) NULL,
    [ManagementPackId] [uniqueidentifier] NOT NULL,
    [Comment] [nvarchar](255) NULL,
    [TimeAdded] [datetime] NOT NULL CONSTRAINT [DF_Staging_RelationshipTypeProperty_TimeAdded]  DEFAULT (getutcdate()),
    [LastModified] [datetime] NOT NULL CONSTRAINT [DF_Staging_RelationshipTypeProperty_LastModified]  DEFAULT (getutcdate()),
    [ModifiedBy] [nvarchar](255) NULL CONSTRAINT [DF_Staging_RelationshipTypeProperty_ModifiedBy]  DEFAULT (user_name()),
    [Required] [bit] NULL,
    [Optimized] [bit] NULL,
    [MinValue] [int] NULL,
    [MaxValue] [int] NULL,
    [RegEx] [nvarchar](255) NULL,
    [EnumTypeId] [uniqueidentifier] NULL,
    [DefaultValue] [nvarchar](2000) NULL,
    CONSTRAINT [PK_Staging_RelationshipTypeProperty] PRIMARY KEY CLUSTERED ([RelationshipPropertyId]),
    CONSTRAINT FK_Staging_RelationshipTypeProperty_RelationshipTypeId FOREIGN KEY (RelationshipTypeId) REFERENCES Staging.RelationshipType (RelationshipTypeId) ,
    CONSTRAINT FK_Staging_RelationshipTypeProperty_ManagementPackId FOREIGN KEY (ManagementPackId) REFERENCES Staging.ManagementPack (ManagementPackId) ,
)
go

if object_id('Staging.ExtractionMP') is null
  create table Staging.ExtractionMP (
    ManagementPackId         uniqueidentifier   NOT NULL
   ,IsDeleted                bit                 NOT NULL
   ,DatasourceId             uniqueidentifier    NOT NULL
   ,TimeAdded                datetime            NOT NULL
   ,LastModified             datetime            NOT NULL
   ,CONSTRAINT [PK_Staging_ExtractionMP] PRIMARY KEY CLUSTERED (ManagementPackId, DatasourceId)
   ,CONSTRAINT [FK_Staging_ExtractionMP_Datasource] FOREIGN KEY (DatasourceId) REFERENCES Staging.Datasource(DatasourceId)
  )
go

if object_id('Staging.ExtractionType') is null
  create table Staging.ExtractionType (
    TypeId                   uniqueidentifier    NOT NULL
   ,TypeName                 nvarchar(255)       NOT NULL
   ,SourceViewName           nvarchar(128)       NULL
   ,IsManagedType            bit                 NOT NULL
   ,IsDeleted                bit                 NOT NULL
   ,IsActive                 bit                 NOT NULL
   ,ManagementPackId         uniqueidentifier    NOT NULL
   ,DatasourceId             uniqueidentifier    NOT NULL
   ,TimeAdded                datetime            NOT NULL
   ,LastModified             datetime            NOT NULL
   ,CONSTRAINT [PK_Staging_ExtractionType] PRIMARY KEY CLUSTERED (TypeId, DatasourceId)
   ,CONSTRAINT [FK_Staging_ExtractionType_ExtractionMP] FOREIGN KEY (ManagementPackId, DatasourceId) REFERENCES Staging.ExtractionMP(ManagementPackId, DatasourceId)
  )
go

if object_id('Staging.ExtractionProperty') is null
  create table Staging.ExtractionProperty (
    PropertyId               uniqueidentifier    NOT NULL
   ,PropertyName             nvarchar(255)       NOT NULL
   ,ColumnName               nvarchar(128)       NOT NULL
   ,IsDeleted                bit                 NOT NULL
   ,IsActive                 bit                 NOT NULL
   ,TypeId                   uniqueidentifier    NOT NULL
   ,DatasourceId             uniqueidentifier    NOT NULL
   ,TimeAdded                datetime            NOT NULL
   ,LastModified             datetime            NOT NULL
   ,CONSTRAINT [PK_Staging_ExtractionProperty] PRIMARY KEY CLUSTERED (PropertyId, TypeId, DatasourceId)
   ,CONSTRAINT [FK_Staging_ExtractionProperty_ExtractionType] FOREIGN KEY (TypeId, DatasourceId) REFERENCES Staging.ExtractionType(TypeId, DatasourceId)
  )
go

if not exists(select 'x' from sys.tables where name = 'EntityTypeExceptionList' and schema_id=SCHEMA_ID('Staging'))
begin
    create table Staging.EntityTypeExceptionList (
        EntityTypeExceptionListId int identity(1, 1) primary key,
	    DatasourceId uniqueidentifier null,
	    BaseManagedTypeId uniqueidentifier not null,
	    TypeName nvarchar (255) not null
    )
end
go

if not exists (select 'x' from sys.indexes where name = 'NCI_EntityTypeExceptionList1')
begin
    create nonclustered index NCI_EntityTypeExceptionList1 on Staging.EntityTypeExceptionList (DatasourceId, BaseManagedTypeId) include (EntityTypeExceptionListId)
end
go


